Package: mritc
Title: MRI Tissue Classification
Date: 2026-08-03
Version: 0.6.1
Authors@R: c(person("Dai", "Feng", role="aut"),
             person("Luke", "Tierney", role="aut"),
             person("Jon", "Clayden", role=c("cre","aut"),
        email="code@clayden.org", comment=c(ORCID="0000-0002-6608-0619")))
Description: Implements various methods for tissue classification in magnetic
        resonance (MR) images of the brain, including normal mixture models
        and hidden Markov normal mixture models, as outlined in Feng &
        Tierney (2011) <doi:10.18637/jss.v044.i07>. These methods allow a
        structural MR image to be classified into gray matter, white matter
        and cerebrospinal fluid tissue types.
Imports: methods, oro.nifti
Suggests: lattice, RNifti, misc3d, tinytest, covr
License: GPL
URL: https://github.com/jonclayden/mritc
BugReports: https://github.com/jonclayden/mritc/issues
NeedsCompilation: yes
Packaged: 2026-08-03 14:59:34 UTC; jon
Author: Dai Feng [aut],
  Luke Tierney [aut],
  Jon Clayden [cre, aut] (ORCID: <https://orcid.org/0000-0002-6608-0619>)
Maintainer: Jon Clayden <code@clayden.org>
Repository: CRAN
Date/Publication: 2026-08-04 14:10:12 UTC

useDynLib(mritc, .registration = TRUE)

export(readMRI, writeMRI, initOtsu, initProp, makeMRIspatial,
       rnormmix, emnormmix, mritc, mritc.em, mritc.icm, mritc.hmrfem,
       mritc.pvhmrfem, mritc.bayes, measureMRI)

S3method(print, mritc)
S3method(summary, mritc)
S3method(plot, mritc)

import(oro.nifti)

importFrom("graphics", "plot")
importFrom("stats", "dnorm", "pnorm", "qnorm", "quantile", "rgamma",
           "rnorm", "runif", "sd")
importFrom("methods", "as")

<!-- README.md is generated from README.Rmd. Please edit that file -->



[![CRAN version](https://www.r-pkg.org/badges/version/mritc)](https://cran.r-project.org/package=mritc) [![CI](https://github.com/jonclayden/mritc/actions/workflows/ci.yaml/badge.svg)](https://github.com/jonclayden/mritc/actions/workflows/ci.yaml) [![codecov](https://codecov.io/gh/jonclayden/mritc/graph/badge.svg?token=C0yDWRkldA)](https://app.codecov.io/gh/jonclayden/mritc) [![Dependencies](https://tinyverse.netlify.app/badge/mritc)](https://cran.r-project.org/package=mritc)

# mritc: MRI Tissue Classification in R

`mritc` classifies the voxels of a structural brain magnetic resonance image into cerebrospinal fluid (CSF), gray matter (GM) and white matter (WM), using normal mixture models and hidden Markov normal mixture models. Several fitting methods are provided, from a plain EM algorithm through to Markov-chain Monte Carlo approaches that also address the partial volume effect (where a voxel contains a mixture of tissue types) and intensity non-uniformity (smoothly varying bias in the scanner signal). The methods are described in [Feng & Tierney (2011)](https://doi.org/10.18637/jss.v044.i07).

## Installation

You can install `mritc` from CRAN with


``` r
install.packages("mritc")
```

or the development version from GitHub with


``` r
# install.packages("remotes")
remotes::install_github("jonclayden/mritc")
```

## Usage

The package includes a sample T1-weighted structural image and a corresponding brain mask, which we can use to demonstrate tissue classification.


``` r
library(mritc)

t1 <- readMRI(system.file("extdata", "t1.rawb.gz", package="mritc"),
              dim=c(91,109,91), format="rawb.gz")
mask <- readMRI(system.file("extdata", "mask.rawb.gz", package="mritc"),
                 dim=c(91,109,91), format="rawb.gz")

tc <- mritc(t1, mask, method="ICM", verbose=FALSE)
tc
## Using method ICM, among 237067 voxels, 
## 18% are classified to CSF,
## 45% are classified to GM,
## 37% are classified to WM.
```

In this case the images are stored in a simple binary format, requiring the dimensions to be specified, but `readMRI()` can also handle NIfTI or ANALYZE files, or numeric arrays read by other packages can be passed in.

The `mritc()` function is an interface to the various classification methods available within the package, selected with its "method" argument. The mask limits classification to within the brain.

The `summary()` method gives a little more detail on the fitted intensity distribution for each tissue type:


``` r
summary(tc)
## Using method ICM, the results are: 
##     proportion%   mean standard deviation
## CSF          18  48.02              13.34
## GM           45  94.35              10.48
## WM           37 128.31              10.32
```

The `plot()` method shows the classified image, overlaid with the tissue label at
each voxel (0 = outside the brain mask, 1 = CSF, 2 = GM, 3 = WM). By default since package version 0.6.0, this uses the viewer from [the `RNifti` package](https://github.com/jonclayden/RNifti); previously [`misc3d::slices3d()`](https://gitlab.com/luke-tierney/misc3d) was used, and this remains an option, although it requires system support for Tk.


``` r
plot(tc, interactive=FALSE)
```

![plot of chunk plot](tools/figures/plot-1.png)

### Checking accuracy against ground truth

The package also bundles the "true" partial volume fractions for this image,
which can be compared to the fitted probabilities using `measureMRI()`. This
reports the mean squared error, misclassification rate, relative volume
error and Dice similarity for each tissue type, and a confusion table.


``` r
csf <- readMRI(system.file("extdata", "csf.rawb.gz", package="mritc"),
               dim=c(91,109,91), format="rawb.gz")
gm <- readMRI(system.file("extdata", "gm.rawb.gz", package="mritc"),
              dim=c(91,109,91), format="rawb.gz")
wm <- readMRI(system.file("extdata", "wm.rawb.gz", package="mritc"),
              dim=c(91,109,91), format="rawb.gz")
truth <- cbind(csf[mask==1], gm[mask==1], wm[mask==1]) / 255

measureMRI(actual=truth, pre=tc$prob)
## $mse
## [1] 0.03309548
## 
## $misclass
## [1] 0.09494362
## 
## $rseVolume
## [1] 0.04655948 0.03611199 0.02440557
## 
## $DSM
## [1] 0.9139096 0.8968206 0.9111253
## 
## $conTable
##              [,1]       [,2]       [,3]
## [1,] 0.9351851852 0.04197286 0.00000000
## [2,] 0.0639774141 0.88062756 0.07775644
## [3,] 0.0008374007 0.07739958 0.92224356
```

See `?mritc` for the other fitting methods available, and `?mritc-package`
for more background on the underlying models.

summary.mritc <- function(object, ...){
    n <- nrow(object$prob)
    k <- ncol(object$prob)
    class <- max.col(object$prob, ties.method="first")
    prop <- sapply(1:k, function(i) sum(class==i)/n)
    prop <- round(prop , 2) * 100
    prop[k] <- 100 - sum(prop[1:(k-1)])
    if(k==5){
        prop.new <- prop
        prop.new[2] <- prop[3]
        prop.new[3] <- prop[5]
        prop.new[4] <- prop[2]
        prop.new[5] <- prop[4]
        prop=prop.new
    }

    result <- matrix(0, nrow=k, ncol=3)
    result[,1] <- prop
    result[,2][1:3] <- round(object$mu, 2)
    result[,3][1:3] <- round(object$sigma, 2)
    if(k==3){
        rownames(result) <- c("CSF", "GM", "WM")
    }
    else{
        result[,2][4:k] <- NA
        result[,3][4:k] <- NA
        if(k==5){
            rownames(result) <- c("CSF", "GM", "WM", "CG", "GW")
        }
        else{
            rownames(result) <- NULL
        }
    }
    colnames(result) <- c(paste("proportion", "%", sep=""), "mean", "standard deviation")
    cat(paste("Using method ", object$method, ", the results are: \n", sep=""))
    print(result)
}

# print.mritc() and summary.mritc() report per-tissue-class proportions
# (and, for summary, fitted means/SDs) derived from a fitted "mritc"
# object's fuzzy membership matrix. summary.mritc() invisibly returns the
# table it prints, which is used below to check values without having to
# parse formatted text.

set.seed(61)
n <- 20
prob3 <- matrix(0, n, 3)
cls3 <- c(rep(1,2), rep(2,12), rep(3,6))  # 10%, 60%, 30%
for (i in 1:n) {
    p <- runif(3, 0.01, 0.05)
    p[cls3[i]] <- 0.6
    prob3[i,] <- p / sum(p)
}
stopifnot(all(max.col(prob3) == cls3))

r3 <- list(prob=prob3, mu=c(10,20,30), sigma=c(1,1,1), method="EM")
class(r3) <- "mritc"

pout <- capture.output(print(r3))
expect_true(any(grepl("^10% are classified to CSF,$", pout)))
expect_true(any(grepl("^60% are classified to GM,$", pout)))
expect_true(any(grepl("^30% are classified to WM\\.$", pout)))

s3 <- capture.output(out3 <- summary(r3))
expect_equal(unname(out3[,"proportion%"]), c(10, 60, 30))
expect_equal(unname(out3[,"mean"]), c(10, 20, 30))
expect_equal(unname(out3[,"standard deviation"]), c(1, 1, 1))
expect_equal(rownames(out3), c("CSF","GM","WM"))

# k=5 (a PVHMRFEM-style result): summary.mritc() re-orders the 5 raw
# probability columns (CSF, CSF-GM, GM, GM-WM, WM) into
# (CSF, GM, WM, CSF-GM, GM-WM) to align proportions against the
# length-3 mu/sigma of the 3 "pure" classes; check this reordering
# against print.mritc()'s un-reordered proportions, which are taken
# directly from the raw column order.
prob5 <- matrix(0, n, 5)
cls5 <- c(rep(1,2), rep(2,3), rep(3,8), rep(4,4), rep(5,3))  # 10,15,40,20,15
for (i in 1:n) {
    p <- runif(5, 0.01, 0.05)
    p[cls5[i]] <- 0.6
    prob5[i,] <- p / sum(p)
}
stopifnot(all(max.col(prob5) == cls5))

r5 <- list(prob=prob5, mu=c(10,20,30), sigma=c(1,1,1), method="PVHMRFEM")
class(r5) <- "mritc"

pout5 <- capture.output(print(r5))
expect_true(any(grepl("^10% are classified to CSF,$", pout5)))
expect_true(any(grepl("^15% are classified to CG,$", pout5)))
expect_true(any(grepl("^40% are classified to GM,$", pout5)))
expect_true(any(grepl("^20% are classified to GW,$", pout5)))
expect_true(any(grepl("^15% are classified to WM\\.$", pout5)))

s5 <- capture.output(out5 <- summary(r5))
expect_equal(rownames(out5), c("CSF","GM","WM","CG","GW"))
expect_equal(unname(out5[,"proportion%"]), c(10, 40, 15, 15, 20))
expect_equal(unname(out5[,"mean"]), c(10, 20, 30, NA, NA))

# --- max.col() tie-breaking determinism ----------------------------------
# Regression test: print.mritc()/summary.mritc()/plot.mritc()/measureMRI()
# discretise the fuzzy membership matrix with max.col(), which by default
# breaks ties at random. Whenever two classes are exactly tied for a
# voxel, the reported proportions used to change from call to call
# depending on the RNG state.
tied <- matrix(c(0.5,0.5,0,
                  0.9,0.05,0.05,
                  0.05,0.9,0.05,
                  0.05,0.05,0.9), ncol=3, byrow=TRUE)
rtie <- list(prob=tied, mu=c(1,2,3), sigma=c(1,1,1), method="EM")
class(rtie) <- "mritc"

set.seed(1); out.a <- capture.output(print(rtie))
set.seed(4); out.b <- capture.output(print(rtie))
expect_identical(out.a, out.b)

set.seed(1); capture.output(sum.a <- summary(rtie))
set.seed(4); capture.output(sum.b <- summary(rtie))
expect_identical(sum.a, sum.b)

rnormmix <- function(n, prop, mu, sigma){

    k <- length(prop)
    if (!(k == length(mu) && k == length(sigma)))
        stop("The dimensions of 'prop', 'mu' and 'sigma' do NOT match.")
    if (!(all(prop > 0) && all(prop < 1)))
        stop("'prop' has to be between 0 and 1")
    if (! isTRUE(all.equal(sum(prop), 1)))
        stop("Sum of 'prop' has to be 1")
    if (!all(sigma > 0))
        stop("All 'sigma's have to be positive")

    c <- sample(1:k, size=n, replace=TRUE, prob=prop)
    y <- rep(0, n)
    for(i in 1:k){
        ci <- c==i
        y[ci] <- rnorm(sum(ci), mean=mu[i], sd=sigma[i])
    }
    cbind(y,c)
}

# rnormmix() draws samples from a normal mixture, returning a two-column
# matrix of (value, component-label) pairs.

set.seed(1)
n <- 5000
prop <- c(0.2, 0.3, 0.5)
mu <- c(-10, 0, 10)
sigma <- c(1, 1, 1)
out <- rnormmix(n, prop, mu, sigma)

expect_equal(dim(out), c(n, 2))
expect_equal(colnames(out), c("y", "c"))
expect_true(all(out[,"c"] %in% 1:3))

# empirical proportions and per-component means/sds should be close to the
# generating parameters
tab <- table(out[,"c"]) / n
expect_equal(as.numeric(tab), prop, tolerance=0.05)
for (i in 1:3) {
    vals <- out[out[,"c"]==i, "y"]
    expect_equal(mean(vals), mu[i], tolerance=0.5)
    expect_equal(sd(vals), sigma[i], tolerance=0.2)
}

# input validation: mismatched lengths
expect_error(rnormmix(10, prop=c(0.5,0.5), mu=c(1,2,3), sigma=c(1,1,1)))
expect_error(rnormmix(10, prop=c(0.5,0.5), mu=c(1,2), sigma=c(1,1,1)))

# prop must be valid
expect_error(rnormmix(10, prop=c(0.5,0.6), mu=c(1,2), sigma=c(1,1)))
expect_error(rnormmix(10, prop=c(0,1), mu=c(1,2), sigma=c(1,1)))

# sigma must be positive
expect_error(rnormmix(10, prop=c(0.5,0.5), mu=c(1,2), sigma=c(1,-1)))

readMRI <- function(file, dim=NULL, format) {
    if(format == "rawb.gz"){
        if(! is.null(dim)){
            if(!is.vector(dim) || length(dim) != 3)
                stop("The dimensions of the file are wrong.")
        }
        f <- gzfile(file, open="rb")
        on.exit(close(f))
        data <- readBin(f,"integer", prod(dim), size = 1, signed = FALSE)
        array(data, dim)
    }
    else if(format == "analyze"){
        readANALYZE(file)@.Data
    }
    else if(format == "nifti"){
        readNIfTI(file)@.Data
    }
    else
        stop("The 'format' is not supported.")
}

writeMRI <- function(data, file, header=NULL, format) {

    objClass <- class(data)
    if(!((objClass=="array" &&
         (length(dim(data)) ==3 || length(dim(data)) == 4 && dim(data)[4] == 1)) ||
         objClass %in% c("nifti", "anlz")))
        stop("'data' has to be an array of dimension 3 or dimension 4 with the
             forth dimension equal to 1, an object of class 'nifti', or
             an object of class 'anlz'.")
    if(format == "rawb.gz"){
        f <- gzfile(file, open="wb")
        on.exit(close(f))
        writeBin(as.vector(if(objClass=="array") data else data@.Data), f, size=1)
    }
    else if(format == "analyze"){
        switch(objClass,
               array=writeANALYZE(as(data, "anlz"), file),
               anlz=writeANALYZE(data, file),
               nifti=writeANALYZE(as(data, "anlz"), file))
    }
    else if(format == "nifti"){
        switch(objClass,
               array=writeNIfTI(as(data, "nifti"), file),
               anlz=writeNIfTI(as(data, "nifti"), file),
               nifti=writeNIfTI(data, file))
    }
    else
        stop("The 'format' is not supported.")   
       
}

# readMRI()/writeMRI() read and write MR images in "rawb.gz" (raw bytes,
# gzip-compressed), "analyze" and "nifti" formats.

set.seed(51)
arr <- array(sample(0:255, 4*4*4, replace=TRUE), dim=c(4,4,4))

# rawb.gz round trip
tmp <- tempfile(fileext=".rawb.gz")
writeMRI(arr, tmp, format="rawb.gz")
arr2 <- readMRI(tmp, dim=c(4,4,4), format="rawb.gz")
expect_equal(as.vector(arr2), as.vector(arr))
expect_equal(dim(arr2), c(4,4,4))
unlink(tmp)

# readMRI validates its 'dim' argument for rawb.gz
tmp2 <- tempfile(fileext=".rawb.gz")
writeMRI(arr, tmp2, format="rawb.gz")
expect_error(readMRI(tmp2, dim=c(4,4), format="rawb.gz"))
unlink(tmp2)

# unsupported format
expect_error(readMRI(tmp2, dim=c(4,4,4), format="bogus"))
expect_error(writeMRI(arr, tempfile(), format="bogus"))

# writeMRI validates the shape/class of 'data'
expect_error(writeMRI(array(1, dim=c(4,4)), tempfile(fileext=".rawb.gz"), format="rawb.gz"))
expect_error(writeMRI(1:10, tempfile(fileext=".rawb.gz"), format="rawb.gz"))

# a 4-D array whose 4th dimension is 1 is also accepted
arr4d <- array(arr, dim=c(4,4,4,1))
tmp3 <- tempfile(fileext=".rawb.gz")
expect_silent(writeMRI(arr4d, tmp3, format="rawb.gz"))
arr4d.back <- readMRI(tmp3, dim=c(4,4,4), format="rawb.gz")
expect_equal(as.vector(arr4d.back), as.vector(arr))
unlink(tmp3)

# read the package's bundled example data

maskfile <- system.file("extdata/mask.rawb.gz", package="mritc")
mask <- readMRI(maskfile, dim=c(91,109,91), format="rawb.gz")
expect_equal(dim(mask), c(91,109,91))
expect_true(all(mask %in% c(0,1)))
expect_true(sum(mask) > 0)

initalBias <- function(y, indices, mu, nvert, blocks, neighbors, nneighbors, weights){
    bias <- y - (indices %*% mu)[-(nvert+1)]
    bias <- c(bias,0)
    for(i in 1:length(blocks)){
        points <- blocks[[i]]
        mat <- matrix(bias[neighbors[points,]], nrow=length(points))
        mat <- mat %*% weights
        bias[points] <- (mat +  bias[points]) /
          (nneighbors[points] + 1)
    }

    bias

}

MCMCSubclass <- function(indices, niter, subvox){
    ind <- indices[t(subvox),]
    nmask <- nrow(subvox)
    p <- matrix(0, nrow=nmask,ncol=ncol(indices))
    for(i in 1:8){
        a <- seq(1,(nmask*8)-7,by=8) + i - 1
        p <- p + ind[a,]
    }
    p.vox <- p/8/niter

    ind <- indices[t(subvox),]
    mode <- matrix(max.col(ind),nrow=8)
    class.modemean=cbind(colSums(mode==1),colSums(mode==2), colSums(mode==3))/8

    list(fussy.vox = p.vox, class.modemean=class.modemean)
}

updateY <- function(yobs, indices, subvox, mu, sigma)
    .Call("updateY", yobs, subvox, indices, mu, sigma)

updateYBias <- function(yobs, bias, indices, subvox, mu, sigma)
    .Call("updateYBias", yobs, bias, subvox, indices, mu, sigma)

getDenSub <- function(y, mu, sigma)
    .Call("getDenSub", y, mu, sigma)

getDenSubBias <- function(y, bias, mu, sigma)
    .Call("getDenSubBias", y, bias, mu, sigma)

updateIndices <- function(neighbors, nneigh, blocks, nblocks, k, indices, check,den){
    .Call("updateIndices", blocks, neighbors, nneigh, k, indices, check, den)
}

updateMusB <- function(Nj, S1j, sigma, eta, xi, k){
    oldtau <- 1/(sigma^2)
    newpre <- eta + oldtau * Nj
    newmean <- (eta*xi + oldtau * S1j) / newpre
    mu <- rnorm(k, mean=newmean, sd=1/sqrt(newpre))
    mu
}

updateSdsB <- function(Nj, ybar, S2j, mu, lambda, phi, k){
    shape <- Nj/2 + lambda
    mid <- S2j + Nj * (ybar - mu)^2
    scale <- 2 * phi / (2 + phi*mid)
    newtau <- rgamma(k, shape=shape, scale=scale)
    sigma <- 1/sqrt(newtau)
    sigma
}

yIndicesSummaries <- function(y, indices, k)
    structure(.Call("yIndicesSummaries", y, indices, k), names = c("Nj", "ybar", "S2j"))


yIndicesSummariesBias <- function(y, bias, indices, k)
    structure(.Call("yIndicesSummariesBias", y, bias, indices, k), names = c("Nj", "ybar", "S2j"))



updateSmoothWeights <- function(gamma.a, gamma.b, nedges,
                                neighbors, nneigh, nvertex,
                                bias, weights){
    .Call("updateSmoothWeights", gamma.a, gamma.b, nedges,
          neighbors, nneigh, nvertex, bias, weights)

}

updateBiasInde <- function(y, bias, neighbors, nneigh, weineighbors, indices,
                       mu, sigma, gamma, weights){
    .Call("updateBiasInde", y, bias, neighbors, nneigh, weineighbors, indices,
          mu, sigma, gamma, weights)
}

updateBias <- function(y, bias, blocks, neighbors, nneigh, weineighbors, indices,
                       mu, sigma, gamma, weights){
    .Call("updateBias", y, bias, blocks, neighbors, nneigh, weineighbors, indices,
          mu, sigma, gamma, weights)
}

updateDistanceUnit <- function(ubsigma.omega, nneigh, nvert, gamma, sigma.omega, sdomega, bias, neighbors, weights, weineighbors, neiDiscrete){

    sigma.omega.new <- qnorm(runif(1, pnorm(0, sigma.omega, sdomega),
                                    pnorm(ubsigma.omega, sigma.omega, sdomega)),
                              sigma.omega, sdomega)
    weights.new <- getWeightsMRI(nnei=nneigh, sigma=sigma.omega.new)
    weineighbors.new <-  rowSums(neiDiscrete %*% weights.new)

    qq <- .Call("updateDistanceUnit", bias, neighbors, nvert, nneigh, weights, weights.new, weineighbors, weineighbors.new, neiDiscrete)
    ratio <- gamma/2*(qq[1] - qq[2]) +
             log(pnorm(ubsigma.omega, sigma.omega, sdomega)-pnorm(0, sigma.omega, sdomega)) -
             log(pnorm(ubsigma.omega, sigma.omega.new, sdomega)-pnorm(0, sigma.omega.new, sdomega))
    if(! is.na(ratio)){
        if(ratio > log(runif(1))){
            sigma.omega <- sigma.omega.new
            weights <- weights.new
            weineighbors <- weineighbors.new
        }
    }
    list(sigma.omega=sigma.omega, weights=weights, weineighbors=weineighbors)


}

mritc.bayes.nobias <- function(y, neighbors, blocks, sub, subvox,
                        spatialMat, beta, mu, sigma, niter, verbose){
    #r <- range(yobs)
    r <- 256
    xi <- r/2
    m1 <- 0.3
    eta <- m1/r^2
    lambda <- m1
    phi <- 1/r^2

    checkErrors(mu=mu, sigma=sigma, err=NULL)

    k <- length(mu)

    nneigh <- ncol(neighbors)
    nblocks <- length(blocks)
    neighbors <- structure(as.integer(neighbors), dim = dim(neighbors))

    y <- as.double(y)

    check <- matrix(as.double(getCheck(nneigh, k, beta, spatialMat)), ncol=k)

    if(sub==F){
        yunique <- sort(unique(y))
        n.yunique <- length(yunique)
        nvert <- length(y)
        ymatch <- match(y, yunique)
        indices <- initialIndices(y, nvert, mu, sigma, k, sub=FALSE)
    }

    else{
        yobs <- y
        nobs <- length(yobs)
        nvert <- nobs*8
        mu <- mu/8
        sigma <- sqrt(sigma^2)/8
        y <- rep(0, nvert)
        y[subvox] <- rep(yobs/8, 8)
        xi <- xi/8
        eta <- eta*64
        lambda <- lambda
        phi <- phi*64
        indices <- initialIndices(yobs, nvert, mu, sigma, k, sub=TRUE, subvox)
    }


    indicesMCMC <- matrix(0L, nrow=nrow(indices), ncol=ncol(indices))
    musave <- matrix(0, ncol=k, nrow=niter)
    sigmasave <- matrix(0, ncol=k, nrow=niter)

    for (i in 1:niter){
        if(sub==F)
            den <- getDen(yunique, n.yunique, ymatch, mu, sigma)
        else den <- getDenSub(y, mu, sigma)

        indices <- updateIndices(neighbors, nneigh, blocks, nblocks, k, indices,
                           check, den)

        indicesMCMC <- .Call("updateCounts", indicesMCMC, indices)

        if(sub==T){
            y <- updateY(yobs, indices, subvox, mu, sigma)
        }

        yIndicesSums <- yIndicesSummaries(y, indices, k)
        Nj <- yIndicesSums$Nj
        ybar <- yIndicesSums$ybar
        S2j <- yIndicesSums$S2j
        S1j <- Nj * ybar
        mu <- updateMusB(Nj, S1j, sigma, eta, xi, k)
        sigma <- updateSdsB(Nj, ybar, S2j, mu, lambda, phi, k)

        musave[i,] <- mu
        sigmasave[i,] <- sigma

                
        if(verbose){
            percent <- i / (niter) * 10
            if(percent %% 1 == 0)
              cat(paste(percent*10, "%", " of iterations has finished", "\n", sep=""))
        }

    }
    if(sub==FALSE)
        prob <- indicesMCMC[-(nvert+1),]/niter
    else
        prob <- class <- MCMCSubclass(indicesMCMC[-(nvert+1),], niter, subvox)$fussy.vox
    list(prob=prob, mu=colMeans(musave), sigma=colMeans(sigmasave))
}


mritc.bayes.bias <- function(y, neighbors, blocks, subvox,
                             neighbors.bias, blocks.bias,
                             weineighbors.bias, weights.bias,
                             spatialMat=matrix(c(2,0,-1,0,2,0,-1,0,2), nrow=3),
                             beta=0.3, mu, sigma, niter=1000, verbose=TRUE){
    r <- 256
    xi <- r/2
    m1 <- 0.3
    eta <- m1/r^2
    lambda <- m1
    phi <- 1/r^2
    gamma.a <- gamma.b <- 1
    nInde <- 200
    ubsigma.omega <- 10
    sigma.omega <- 1
    sdomega <- 0.1
    
    checkErrors(mu=mu, sigma=sigma, err=NULL)
    
    k <- length(mu)
    nneigh <- ncol(neighbors)
    nneigh.bias <- ncol(neighbors.bias)
    nblocks <- length(blocks)
    neighbors <- structure(as.integer(neighbors), dim = dim(neighbors))
    neighbors.bias <- structure(as.integer(neighbors.bias), dim = dim(neighbors.bias))
    weineighbors.bias <- structure(as.double(weineighbors.bias), dim = dim(weineighbors.bias))
    nedges.bias <- as.integer(sum(neighbors.bias != nrow(neighbors.bias)+1) / 2)
    
    y <- as.double(y)
    check <- matrix(as.double(getCheck(nneigh, k, beta, spatialMat)), ncol=k)
    yobs <- y
    nobs <- length(yobs)
    nvert <- as.integer(nobs*8)
    mu <- mu/8
    sigma <- sqrt(sigma^2)/8
    y <- rep(0, nvert)
    y[subvox] <- rep(yobs/8, 8)
    xi <- xi/8
    eta <- eta*64
    lambda <- lambda
    phi <- phi*64
    neiDiscrete <- (neighbors.bias < nvert+1)
    neiDiscrete <- structure(as.integer(neiDiscrete), dim=dim(neiDiscrete))
    indices <- initialIndices(yobs, nvert, mu, sigma, k, sub=TRUE, subvox)
    bias <- initalBias(y, indices, mu, nvert, blocks.bias, neighbors.bias, weineighbors.bias, weights.bias)
    
    
    indicesMCMC <- matrix(0L, nrow=nrow(indices), ncol=ncol(indices))
    musave <- matrix(0, ncol=k, nrow=niter)
    sigmasave <- matrix(0, ncol=k, nrow=niter)
    
    sigmaomegasave <- rep(0, niter)
    ysave <- rep(0, length(y))
    biassave <- rep(0, length(bias))
    for (i in 1:(niter+nInde)){
        den <- getDenSubBias(y, bias, mu, sigma)

        indices <- updateIndices(neighbors, nneigh, blocks, nblocks, k, indices,
                                 check, den)
    
        if(i > nInde){
            indicesMCMC <- .Call("updateCounts", indicesMCMC, indices)
        }
    
        y <- updateYBias(yobs, bias, indices, subvox, mu, sigma)
    
        yIndicesSums <- yIndicesSummariesBias(y, bias, indices, k)
        Nj <- yIndicesSums$Nj
        ybar <- yIndicesSums$ybar
        S2j <- yIndicesSums$S2j
        S1j <- Nj * ybar
        mu <- updateMusB(Nj, S1j, sigma, eta, xi, k)
        sigma <- updateSdsB(Nj, ybar, S2j, mu, lambda, phi, k)
        
        if(i > nInde){
            ysave <- ysave + y
            musave[(i-nInde),] <- mu
            sigmasave[(i-nInde),] <- sigma
        }
        
        gamma <- updateSmoothWeights(gamma.a, gamma.b, nedges.bias,
                                     neighbors.bias,
                                     nneigh.bias, nvert,
                                     bias, weights.bias)
        #if(i > nInde){
        #    gammasave[(i-nInde)] <- gamma
        #}
        
        if(i <= nInde){
            bias <- updateBiasInde(y, bias, neighbors.bias, nneigh.bias,
                                   weineighbors.bias, indices,
                                   mu, sigma, gamma, weights.bias)
        }
        else{
            bias <- updateBias(y, bias, blocks.bias, neighbors.bias, nneigh.bias,
                               weineighbors.bias, indices, mu, sigma, gamma,
                               weights.bias)
            biassave <- biassave + bias
        }
        
        if(i > nInde){
            sigma.omega.old <- sigma.omega
            omega <- updateDistanceUnit(ubsigma.omega, nneigh.bias, nvert,
                                        gamma, sigma.omega,
                                        sdomega, bias, neighbors.bias,
                                        weights.bias, weineighbors.bias,
                                        neiDiscrete)
            sigma.omega <- omega$sigma.omega
            weights.bias <- omega$weights
            weineighbors.bias <- omega$weineighbors
            
            
            sigmaomegasave[(i-nInde)] <- sigma.omega
        }
        
        if(verbose){
            percent <- i / (niter+nInde) * 10
            if(percent %% 1 == 0)
              cat(paste(percent*10, "%", " of iterations has finished", "\n", sep=""))
        }
    }
    
    prob <- class <- MCMCSubclass(indicesMCMC[-(nvert+1),], niter, subvox)$fussy.vox
    #list(prob=prob, mu=colMeans(musave), sigma=colMeans(sigmasave))
    list(prob=prob, indices=indicesMCMC/niter,
         mu=colMeans(musave), sigma=colMeans(sigmasave),
         y=ysave/niter, bias=biassave/niter,
         xi=xi, eta=eta, lambda=lambda, phi=phi,
         gamma.a=gamma.a, gamma.b=gamma.b,
         ubsigma.omega=ubsigma.omega, sigma.omega.ini=mean(sigmaomegasave))
}

mritc.bayes <- function(y, neighbors, blocks, sub, subvox,
                        subbias=FALSE, neighbors.bias=NULL, blocks.bias=NULL,
                        weineighbors.bias=NULL, weights.bias=NULL,
                        spatialMat=(if(sub) matrix(c(2,0,-1,0,2,0,-1,0,2), nrow=3)
                                    else diag(1,3)),
                        beta=ifelse(sub, 0.3, 0.7), mu, sigma, niter=100, verbose=TRUE){
    if(subbias==FALSE){
        mritc.bayes.nobias(y, neighbors, blocks, sub, subvox,
                           spatialMat, beta, mu, sigma, niter, verbose)
    }
    else{
        if(sub==FALSE){
            stop("PV and intensity non-uniformity have to be addressed simultaneously!")
        }
        else{
            mritc.bayes.bias(y, neighbors, blocks, subvox,
                             neighbors.bias, blocks.bias,
                             weineighbors.bias, weights.bias,
                             spatialMat, beta, mu, sigma, niter, verbose)
        }
    }
    
}

# mritc.bayes() fits the hierarchical Bayesian hidden Markov normal mixture
# model via MCMC, in three flavours: whole-voxel ("nobias", sub=FALSE),
# higher-resolution sub-voxel (sub=TRUE), and sub-voxel with intensity
# non-uniformity ("bias") correction. Small niter/mask keep these fast.

set.seed(31)
mask <- array(0, dim=c(8,8,8))
mask[2:7,2:7,2:7] <- 1
intarr <- array(rnorm(8^3, 100, 10), dim=c(8,8,8))
intarr[mask==1] <- intarr[mask==1] + rep(c(0,20,40), length.out=sum(mask==1))
y <- intarr[mask==1]
init <- initOtsu(y, 2)

# whole-voxel (sub=FALSE)

sp <- makeMRIspatial(mask, nnei=6, sub=FALSE)
rb <- mritc.bayes(y, sp$neighbors, sp$blocks, sub=FALSE, subvox=NULL,
                   mu=init$mu, sigma=init$sigma, niter=20, verbose=FALSE)
expect_equal(dim(rb$prob), c(length(y), 3))
expect_equal(rowSums(rb$prob), rep(1, length(y)), tolerance=1e-6)
expect_true(all(rb$prob >= 0 & rb$prob <= 1))
expect_equal(sort(rb$mu), sort(init$mu), tolerance=10)

# higher-resolution sub-voxel (sub=TRUE)

spsub <- makeMRIspatial(mask, nnei=6, sub=TRUE)
rbsub <- mritc.bayes(y, spsub$neighbors, spsub$blocks, sub=TRUE, subvox=spsub$subvox,
                      mu=init$mu, sigma=init$sigma, niter=20, verbose=FALSE)
# the sub-voxel model still reports one fuzzy membership row per original
# (whole) voxel
expect_equal(dim(rbsub$prob), c(length(y), 3))
expect_equal(rowSums(rbsub$prob), rep(1, length(y)), tolerance=1e-6)

# sub-voxel with bias-field correction

sp26 <- makeMRIspatial(mask, nnei=26, sub=TRUE, bias=TRUE)
rbbias <- mritc.bayes(y, spsub$neighbors, spsub$blocks, sub=TRUE, subvox=spsub$subvox,
                       subbias=TRUE, neighbors.bias=sp26$neighbors, blocks.bias=sp26$blocks,
                       weineighbors.bias=sp26$weineighbors, weights.bias=sp26$weights,
                       mu=init$mu, sigma=init$sigma, niter=5, verbose=FALSE)
expect_equal(dim(rbbias$prob), c(length(y), 3))
expect_equal(rowSums(rbbias$prob), rep(1, length(y)), tolerance=1e-6)

# subbias=TRUE requires the higher-resolution (sub=TRUE) model: the two
# features address partial-volume and bias-field effects simultaneously
# and cannot be decoupled (see mritc.bayes.R)
expect_error(mritc.bayes(y, sp$neighbors, sp$blocks, sub=FALSE, subvox=NULL,
                          subbias=TRUE, neighbors.bias=sp26$neighbors, blocks.bias=sp26$blocks,
                          weineighbors.bias=sp26$weineighbors, weights.bias=sp26$weights,
                          mu=init$mu, sigma=init$sigma, niter=5, verbose=FALSE))

# mu/sigma are validated via checkErrors()
expect_error(mritc.bayes(y, sp$neighbors, sp$blocks, sub=FALSE, subvox=NULL,
                          mu=c(1,2,3), sigma=c(1,-1,1), niter=5, verbose=FALSE))

emnormmix <- function(y, prop, mu, sigma, err, maxit, verbose){
    checkErrors(prop, mu, sigma, err)

    if (length(err) < 3) err <- rep(err, length.out = 3)

    k <- length(prop)
    ## tabulate distinct y values
    yvals <- sort(unique(y))
    n.yvals <- length(yvals)
    counts <- as.vector(table(y))

    ## enlarge y to dy.
    dy <- rep(yvals,each=k)
    no <- length(y)

    it <- 0

    repeat {
        it <- it + 1
        muold <- mu
        sigmaold <- sigma
        propold <- prop

        ## compute new component probabilities
        dprop <- rep(propold, n.yvals)
        dmu <- rep(muold, n.yvals)
        dsigma <- rep(sigmaold, n.yvals)
        dn <- dnorm(dy, mean = dmu, sd = dsigma)
        interprop <- matrix(dprop * dn, ncol = k, byrow = TRUE)
        indices <- sweep(interprop, 1, rowSums(interprop), `/`)

        ## compute new weights
        sumIndices <- as.vector(counts %*% indices)
        prop <- sumIndices/no

        ## compute new means
        mu <- as.vector(((yvals * counts) %*% indices)/sumIndices)

        ## compute new variances
        sigma <- sqrt(counts %*% (outer(yvals, mu, `-`) ^ 2 * indices) / sumIndices)

        flag <- checkStopVerbose(muold, mu, sigmaold, sigma, err, it, maxit, verbose, propold, prop)
        if(flag==1) break

      
    }
    list(prop = prop, mu = mu, sigma = sigma)
}


#############################################################
mritc.em <- function(y, prop, mu, sigma, err=1e-4, maxit=200, verbose) {

    checkErrors(prop, mu, sigma, err)

    k <- length(prop)
    nvert <- length(y)

    #get the EM estimation
    result <- emnormmix(y, prop, mu, sigma, err, maxit, verbose)
    prop <- result$prop
    mu <- result$mu
    sigma <- result$sigma


    ## tabulate distinct y values
    yvals <- sort(unique(y))
    n.yvals <- length(yvals)
    counts <- as.vector(table(y))

    ## k : number of normal distributions in the mixture.
    k <- length(prop)

    ## enlarge y to dy.
    dy <- rep(yvals,each=k)
    no <- length(y)

    ## compute component probabilities
    dprop <- rep(prop, n.yvals)
    dmu <- rep(mu, n.yvals)
    dsigma <- rep(sigma, n.yvals)
    dn <- dnorm(dy, mean = dmu, sd = dsigma)
    interprop <- matrix(dprop * dn, ncol = k, byrow = TRUE)
    indices <- sweep(interprop, 1, rowSums(interprop), `/`)

    pre <- indices

    list(prob=pre[match(y, yvals),], mu=mu, sigma=sigma)
}

# mritc.em() (and its internal workhorse emnormmix()) fit a normal mixture
# model to a vector of voxel intensities via the EM algorithm, with no
# spatial (Markov random field) component.

set.seed(21)
y <- c(rnorm(300, 10, 1), rnorm(300, 20, 1), rnorm(300, 30, 1))
init <- initOtsu(y, 2)

r <- mritc.em(y, init$prop, init$mu, init$sigma, verbose=FALSE)
expect_equal(dim(r$prob), c(length(y), 3))
expect_equal(rowSums(r$prob), rep(1, length(y)), tolerance=1e-6)
expect_true(all(r$prob >= 0 & r$prob <= 1))
expect_equal(sort(r$mu), c(10, 20, 30), tolerance=0.5)
expect_true(all(r$sigma > 0))

# each voxel's most probable class agrees with the mixture component it was
# actually generated from, for this well-separated case
truth <- rep(1:3, each=300)
classified <- max.col(r$prob)
# allow the label permutation to differ, but classes should still align
tab <- table(truth, classified)
expect_equal(sum(apply(tab, 1, max)) / length(y), 1, tolerance=0.01)

# input validation is delegated to checkErrors()
expect_error(mritc.em(y, prop=c(0.5,0.5), mu=c(1,2,3), sigma=c(1,1,1), verbose=FALSE))
expect_error(mritc.em(y, prop=init$prop, mu=init$mu, sigma=c(-1,1,1), verbose=FALSE))

# emnormmix() is the internal engine used by mritc.em(); mritc.em() adds
# the per-voxel 'prob' matrix on top of the same (prop, mu, sigma) fit
em <- mritc:::emnormmix(y, init$prop, init$mu, init$sigma, err=1e-4, maxit=200, verbose=FALSE)
expect_equal(em$mu, r$mu)
expect_equal(em$sigma, r$sigma)

# --- mritc.icm / mritc.hmrfem / mritc.pvhmrfem --------------------------
# These add a spatial (Potts model) smoothing term, so need a mask and a
# neighbour structure.

set.seed(22)
mask <- array(0, dim=c(8,8,8))
mask[2:7,2:7,2:7] <- 1
intarr <- array(rnorm(8^3, 100, 10), dim=c(8,8,8))
intarr[mask==1] <- intarr[mask==1] + rep(c(0,20,40), length.out=sum(mask==1))
yv <- intarr[mask==1]
initv <- initOtsu(yv, 2)
sp <- makeMRIspatial(mask, nnei=6, sub=FALSE)

ricm <- mritc.icm(yv, sp$neighbors, sp$blocks, mu=initv$mu, sigma=initv$sigma, verbose=FALSE)
expect_equal(dim(ricm$prob), c(length(yv), 3))
expect_equal(rowSums(ricm$prob), rep(1, length(yv)), tolerance=1e-6)
expect_equal(sort(ricm$mu), sort(initv$mu), tolerance=5)

rhmrf <- mritc.hmrfem(yv, sp$neighbors, sp$blocks, mu=initv$mu, sigma=initv$sigma, verbose=FALSE)
expect_equal(dim(rhmrf$prob), c(length(yv), 3))
expect_equal(rowSums(rhmrf$prob), rep(1, length(yv)), tolerance=1e-6)

rpv <- mritc.pvhmrfem(yv, sp$neighbors, sp$blocks, mu=initv$mu, sigma=initv$sigma, verbose=FALSE)
# PVHMRFEM models 3 pure classes plus 2 intermediate partial-volume
# classes; i.e., prob has 5 columns
expect_equal(dim(rpv$prob), c(length(yv), 5))
expect_equal(rowSums(rpv$prob), rep(1, length(yv)), tolerance=1e-6)
expect_equal(length(rpv$mu), 3)

# mritc.icm/mritc.hmrfem validate mu/sigma via checkErrors()
expect_error(mritc.icm(yv, sp$neighbors, sp$blocks, mu=c(1,2,3), sigma=c(1,-1,1), verbose=FALSE))
expect_error(mritc.hmrfem(yv, sp$neighbors, sp$blocks, mu=c(1,2,3), sigma=c(1,-1,1), verbose=FALSE))

# mritc.pvhmrfem must likewise reject a non-positive sigma rather than
# silently proceeding into undefined behaviour (regression test: this
# used to skip validation entirely and could crash R on bad input)
expect_error(mritc.pvhmrfem(yv, sp$neighbors, sp$blocks, mu=c(1,2,3), sigma=c(1,-1,1), verbose=FALSE))
expect_error(mritc.pvhmrfem(yv, sp$neighbors, sp$blocks, mu=c(1,2,3), sigma=c(1,0,1), verbose=FALSE))

getWeightsMRI <- function(nnei, sigma=1){
    if(! nnei %in% c(26))
        stop("'nnei' has to be among (26)")

    width <- ((nnei + 1)^(1/3) - 1) /  2
    expand <- t(data.matrix(expand.grid(-width:width,
                                     -width:width,
                                     -width:width)[-(nnei/2 +1),]))
 
    weights <- dnorm(sqrt((expand[1,]/width)^2 + (expand[2,]/width)^2 +
                          (expand[3,]/width)^2), mean=0, sd=sigma)
    weights /sum(weights)
}

# getBlocksMRI(), getNeighborsMRI(), getWeightsMRI() and makeMRIspatial()
# build the spatial (neighbour / checkerboard-block) structures used by the
# Markov random field based classification methods. None of these are
# exported, so they are addressed via ':::'.

mask <- array(0, dim=c(6,6,6))
mask[2:5,2:5,2:5] <- 1
nvox <- sum(mask)

# --- getBlocksMRI -----------------------------------------------------

# a 2-colour checkerboard partitions every masked voxel exactly once
blocks2 <- mritc:::getBlocksMRI(mask, 2)
expect_equal(length(blocks2), 2)
expect_equal(sum(sapply(blocks2, length)), nvox)
expect_equal(sort(unlist(blocks2)), 1:nvox)

# an 8-colour partition does likewise
blocks8 <- mritc:::getBlocksMRI(mask, 8)
expect_equal(length(blocks8), 8)
expect_equal(sum(sapply(blocks8, length)), nvox)
expect_equal(sort(unlist(blocks8)), 1:nvox)

# input validation
expect_error(mritc:::getBlocksMRI(mask, 3))
expect_error(mritc:::getBlocksMRI(array(0, dim=c(6,6,6)), 2))
expect_error(mritc:::getBlocksMRI(array(2, dim=c(6,6,6)), 2))
expect_error(mritc:::getBlocksMRI(matrix(1, 6, 6), 2))

# --- getNeighborsMRI ----------------------------------------------------

nb6 <- mritc:::getNeighborsMRI(mask, 6)
expect_equal(dim(nb6), c(nvox, 6))
# entries are either a valid (1-based) voxel index or nvox+1 (the
# off-mask/"virtual" sentinel index)
expect_true(all(nb6 >= 1 & nb6 <= nvox + 1))

nb18 <- mritc:::getNeighborsMRI(mask, 18)
expect_equal(dim(nb18), c(nvox, 18))

nb26 <- mritc:::getNeighborsMRI(mask, 26)
expect_equal(dim(nb26), c(nvox, 26))

expect_error(mritc:::getNeighborsMRI(mask, 10))

# a voxel in the interior of the masked block (not touching its edge) has
# no off-mask neighbors in the 6-neighbor scheme
ind <- which(mask == 1, arr.ind=TRUE)
idx <- which(ind[,1]==3 & ind[,2]==3 & ind[,3]==3)
expect_equal(length(idx), 1)
expect_true(all(nb6[idx,] <= nvox))

# --- getWeightsMRI --------------------------------------------------------

w <- mritc:::getWeightsMRI(26, sigma=1)
expect_equal(length(w), 26)
expect_equal(sum(w), 1)
expect_true(all(w > 0))
# under an isotropic Gaussian kernel, the 6 face-touching (unit-distance)
# neighbors get equal, and the largest, weight; the 8 corner (sqrt(3))
# neighbors get the smallest
expect_equal(sum(w == max(w)), 6)
expect_equal(sum(w == min(w)), 8)

expect_error(mritc:::getWeightsMRI(6, sigma=1))
expect_error(mritc:::getWeightsMRI(18, sigma=1))

# --- makeMRIspatial -------------------------------------------------------

sp <- makeMRIspatial(mask, nnei=6, sub=FALSE)
expect_equal(dim(sp$neighbors), c(nvox, 6))
expect_equal(length(sp$blocks), 2)
expect_false(sp$sub)
expect_null(sp$subvox)
expect_null(sp$weights)

sp8 <- makeMRIspatial(mask, nnei=26, sub=FALSE)
expect_equal(length(sp8$blocks), 8)

# sub=TRUE splits every voxel into 8 sub-voxels
spsub <- makeMRIspatial(mask, nnei=6, sub=TRUE)
expect_equal(dim(spsub$neighbors), c(nvox*8, 6))
expect_equal(dim(spsub$subvox), c(nvox, 8))
expect_true(spsub$sub)

# bias=TRUE additionally returns per-neighbor weights
spbias <- makeMRIspatial(mask, nnei=26, sub=FALSE, bias=TRUE)
expect_equal(length(spbias$weights), 26)
expect_equal(length(spbias$weineighbors), nvox)

expect_error(makeMRIspatial(mask, nnei=10))

mritc.icm <- function(y, neighbors, blocks, spatialMat=diag(1,3), beta=0.4, mu, sigma, err=1e-4, maxit=20, verbose){

    checkErrors(mu=mu, sigma=sigma, err=err)

    if (length(err) < 2) err <- rep(err, length.out = 2)

    k <- length(mu)
    nvert <- length(y)
    yunique <- sort(unique(y))
    n.yunique <- length(yunique)
    ymatch <- match(y, yunique)
    nvert <- length(y)
    nneigh <- ncol(neighbors)

    check <- getCheck(nneigh, k, beta, spatialMat)
    indices <- initialIndices(y, nvert, mu, sigma, k, sub=FALSE)

    prob <- matrix(0, ncol=k, nrow=nvert)
    it <- 0

    repeat{
        it <- it + 1
        #update indices
        muold <- mu
        sigmaold <- sigma
        den <- getDen(yunique, n.yunique, ymatch, mu, sigma)

        
        for (i in 1:length(blocks)){
            points <- blocks[[i]]
            prob[points,] <- updateProbs(neighbors[points,], nneigh, indices,
                                 den[points,], k, check)
            indices.focus <- max.col(prob[points,])
            indices.focus <- do.call(rbind, lapply(1:k, function(x) indices.focus==x))
            indices[points,] <- t(indices.focus)
        }

        #update mu and sigma
        Nj <- colSums(indices)
        mu <- colSums(y*(indices[-(nvert+1),]))/Nj
        sigma <- sqrt(colSums(
                     (y*(indices[-(nvert+1),])- t(t(indices[-(nvert+1),])*mu))^2)/Nj)

        flag <- checkStopVerbose(muold, mu, sigmaold, sigma, err, it, maxit, verbose)
        if(flag==1) break

    }

    list(prob=prob/rowSums(prob), mu=mu, sigma=sigma)
}

print.mritc <- function(x, ...){
    n <- nrow(x$prob)
    k <- ncol(x$prob)
    class <- max.col(x$prob, ties.method="first")
    prop <- sapply(1:k, function(i) sum(class==i)/n)
    prop <- round(prop , 2) * 100
    prop[k] <- 100 - sum(prop[1:(k-1)])
    if(k==3){
        cat(paste("Using method ", x$method, ", among ", n,
                  " voxels, \n", prop[1], "% are classified to CSF,\n",
                  prop[2], "% are classified to GM,\n", 
                  prop[3], "% are classified to WM.\n", sep=""))
    }
    else{
        if(k==5){
            cat(paste("Using method ", x$method, ", among ", n,
                      " voxels, \n", prop[1], "% are classified to CSF,\n",
                      prop[2], "% are classified to CG,\n",
                      prop[3], "% are classified to GM,\n",
                      prop[4], "% are classified to GW,\n",
                      prop[5], "% are classified to WM.\n", sep=""))

        }
        else{
            cat(paste("Using method ", x$method, ", among ", n,
                      " voxels, \n",
                      "The proportions of different tissue types are: \n",
                      prop, sep=""))
        }
    }
}

mritc <- function(intarr, mask, method=c("EM", "ICM", "HMRFEM", "MCMC",
                                  "MCMCsub", "PVHMRFEM", "MCMCsubbias"),
                   verbose=TRUE){
    if (length(dim(intarr)) != 3)
        stop("The intensity values of an MR image has to be in a 3D array.")

    if(length(dim(mask)) != 3)
        stop("The 'mask' has to be of dimension 3.")
    if(! all(unique(as.vector(mask)) %in% 0:1))
        stop("The value of mask has to be either 1 or 0.")
    if(all(mask==0))
        stop("All voxels are outside the mask.")
    
    method <- match.arg(method)
    
    y <- intarr[mask == 1]
    sub <- (method %in% c("MCMCsub", "MCMCsubbias"))
    mrispatial <- makeMRIspatial(mask, nnei=6, sub)
    if(method=="MCMCsubbias"){
        mrispatial26 <- makeMRIspatial(mask, nnei=26, sub=TRUE, bias=TRUE)
    }
    init <- initOtsu(y, 2)
    prop <- init$prop
    mu <- init$mu
    sigma <- init$sigma
  
    result <- switch(method,
              EM = mritc.em(y, prop, mu, sigma, verbose=verbose),
              ICM = mritc.icm(y, mrispatial$neighbors, mrispatial$blocks,
                          mu=mu, sigma=sigma, verbose=verbose),
              HMRFEM = mritc.hmrfem(y, mrispatial$neighbors, mrispatial$blocks,
                             mu=mu, sigma=sigma, verbose=verbose),
              MCMC = mritc.bayes.nobias(y, mrispatial$neighbors, mrispatial$blocks,
                         mrispatial$sub, mrispatial$subvox,
                         spatialMat=diag(1,3), beta=0.7, mu, sigma,
                         niter=100, verbose=verbose),
              MCMCsub = mritc.bayes.nobias(y, mrispatial$neighbors, mrispatial$blocks,
                            mrispatial$sub, mrispatial$subvox,
                            spatialMat=matrix(c(2,0,-1,0,2,0,-1,0,2), nrow=3),
                            beta=0.3, mu, sigma, niter=100, verbose=verbose),
              MCMCsubbias = mritc.bayes.bias(y, mrispatial$neighbors,
                                mrispatial$blocks, mrispatial$subvox,
                                mrispatial26$neighbors, mrispatial26$blocks,
                                mrispatial26$weineighbors, mrispatial26$weights,
                                spatialMat=matrix(c(2,0,-1,0,2,0,-1,0,2), nrow=3),
                                beta=0.3, mu, sigma, niter=1000, verbose=verbose),
              PVHMRFEM = mritc.pvhmrfem(y, mrispatial$neighbors, mrispatial$blocks,
                                  mu=mu, sigma=sigma, verbose=verbose))
    
    
    class(result) <- "mritc"
    result$method <- method
    result$mask <- mask

    result
}

# mritc() is the top-level convenience wrapper: given a raw intensity array
# and a mask, it derives initial parameter estimates and spatial structures
# automatically and dispatches to the appropriate fitting function.

set.seed(41)
mask <- array(0, dim=c(8,8,8))
mask[2:7,2:7,2:7] <- 1
intarr <- array(rnorm(8^3, 100, 10), dim=c(8,8,8))
intarr[mask==1] <- intarr[mask==1] + rep(c(0,20,40), length.out=sum(mask==1))
nvox <- sum(mask)

for (m in c("EM", "ICM", "HMRFEM", "PVHMRFEM", "MCMC", "MCMCsub")) {
    out <- mritc(intarr, mask, method=m, verbose=FALSE)
    expect_inherits(out, "mritc")
    expect_equal(out$method, m)
    expect_equal(nrow(out$prob), nvox)
    expect_equal(rowSums(out$prob), rep(1, nvox), tolerance=1e-6)
    expect_identical(out$mask, mask)
}

# PVHMRFEM alone produces 5 columns (3 pure + 2 partial-volume classes)
outpv <- mritc(intarr, mask, method="PVHMRFEM", verbose=FALSE)
expect_equal(ncol(outpv$prob), 5)

# MCMCsubbias combines the sub-voxel and bias-field-correction models
outbias <- mritc(intarr, mask, method="MCMCsubbias", verbose=FALSE)
expect_inherits(outbias, "mritc")
expect_equal(nrow(outbias$prob), nvox)
expect_equal(rowSums(outbias$prob), rep(1, nvox), tolerance=1e-6)

# default method is "EM"
outdef <- mritc(intarr, mask, verbose=FALSE)
expect_equal(outdef$method, "EM")

# method can be abbreviated
outabbr <- mritc(intarr, mask, method="IC", verbose=FALSE)
expect_equal(outabbr$method, "ICM")

# verbose defaults to TRUE, and can be silenced
expect_true(length(capture.output(mritc(intarr, mask, method="EM"))) > 0)
expect_silent(mritc(intarr, mask, method="EM", verbose=FALSE))

# input validation
expect_error(mritc(intarr[,,1], mask, method="EM"))
expect_error(mritc(intarr, mask[,,1], method="EM"))
expect_error(mritc(intarr, mask * 0, method="EM"))
expect_error(mritc(intarr, mask * 2, method="EM"))
expect_error(mritc(intarr, mask, method="nonexistent"))

#Update probs
updateProbs <- function(neighbors, nneigh, indices, den, k, check){
    Nij <- matrix(0, nrow=nrow(neighbors), ncol=k)
    for(m in 1:nneigh){
        Nij <- Nij + indices[neighbors[,m],]
    }
    or <- Nij[,1]
    for(i in 2:k)
        or <- or + Nij[,i]*(nneigh+1)^(i-1)
    or <- or + 1
    prob <- den*check[or,]
    prob <- prob/rowSums(prob)
    prob[is.na(rowSums(prob)),] <- rep(1/k, k)
    prob[prob==Inf] <- 1/k
    prob
}

#Update mus
updateMus <- function(prob, y){
    inten <- y * prob
    mu <- colSums(inten)/colSums(prob)
    mu
}

#Update sds
updateSds <- function(prob, y, mu, nvert, k){
    mu <- matrix(mu, nrow=nvert, ncol=k, byrow=T)
    diff2 <- (y-mu)^2*prob
    sigma2 <- colSums(diff2)/colSums(prob)
    sqrt(sigma2)
}

updateIndicesHMRFEM <- function(neighbors, nneigh, blocks, nblocks, k, indices, check, den){
    .Call("updateIndicesHMRFEM", blocks, neighbors, nneigh, k, indices, check, den)
}

plot.mritc <- function(x, method=c("RNifti","misc3d"), ...){
    method <- match.arg(method)
    class <- max.col(x$prob, ties.method="first")
    x$mask[x$mask==1] <- class
    if (method == "misc3d" && requireNamespace("misc3d",quietly=TRUE))
        misc3d::slices3d(x$mask, ...)
    else if (method == "RNifti" && requireNamespace("RNifti",quietly=TRUE)) {
        classification <- x$mask
        RNifti::view(RNifti::lyr(classification, min=0, max=max(class)), ...)
    } else
        stop("The requested viewer package (", method, ") is not available")
    invisible(NULL)
}

mritc.hmrfem <- function(y, neighbors, blocks, spatialMat=diag(1,3), beta=0.5, mu, sigma, err=1e-4, maxit=20, verbose){

    checkErrors(mu=mu, sigma=sigma, err=err)

    if (length(err) < 2) err <- rep(err, length.out = 2)

    k <- length(mu)
    nvert <- length(y)


    nneigh <- ncol(neighbors)
    nblocks <- length(blocks)
    neighbors <- structure(as.integer(neighbors), dim = dim(neighbors))

    yunique <- sort(unique(y))
    n.yunique <- length(yunique)
    nvert <- length(y)
    ymatch <- match(y, yunique)

    muold <- rep(0,k)
    sigmaold <- rep(0,k)
    niter <- 10

    check <- getCheck(nneigh, k, beta, spatialMat)
    indices <- initialIndices(y, nvert, mu, sigma, k, sub=FALSE)

    it <- 0

    repeat{
        it <- it + 1
        muold <- mu
        sigmaold <- sigma
        for(j in 1:niter){
            den <- getDen(yunique, n.yunique, ymatch, mu, sigma)
            indices <- updateIndicesHMRFEM(neighbors, nneigh, blocks, nblocks, k, indices, check, den)
        }
        prob  <- updateProbs(neighbors, nneigh, indices, den, k, check)
        mu <- updateMus(prob, y)
        sigma <- updateSds(prob, y, mu, nvert, k)

        flag <- checkStopVerbose(muold, mu, sigmaold, sigma, err, it, maxit, verbose)
        if(flag==1) break
    }

    list(prob=prob, mu=mu, sigma=sigma)
}

getConTable <- function(actual, pre){
    if(length(actual) != length(pre))
        stop("The length of 'actual' is not equal to the length of 'pre'.")
    aclass <- unique(actual)
    pclass <- unique(pre)
    if(! (all(pclass %in% aclass)))
        stop("The range of the predicted classes is out of the range of the actual classes.")
    class <- sort(aclass)
    nclass <- length(class)
    table <- matrix(0, nrow=nclass, ncol=nclass)
    for(i in 1:nclass){
        focus <- pre[actual == i]
        n <- length(focus)
        for(j in 1:nclass){
            table[i,j] <- sum(focus==j)/n
        }
    }
    t(table)
}

getDSM <- function(actual, pre){
    if(length(actual) != length(pre))
        stop("The length of 'actual' is not equal to the length of 'pre'.")
    aclass <- unique(actual)
    pclass <- unique(pre)
    if(! (all(pclass %in% aclass)))
        stop("The range of the predicted classes is out of the range of the actual classes.")
    class <- sort(aclass)
    nclass <- length(class)
    DSM <- rep(0, nclass)
    for(i in 1:nclass){
        comp <- pre == i
        true <- actual == i
        inter <- sum(comp & true)
        DSM[i] <- 2 * inter / (sum(comp) + sum(true))
    }
    DSM
}


measureMRI <- function(intvec=NULL, actual, pre){
    if (!all(dim(actual) == dim(pre)))
        stop("The dimension of 'actual' does not match that of the 'pre'.")
    if (!is.null(intvec) && !is.vector(intvec))
        stop("'intvec' has to be a vector.")
    if (!is.null(intvec) && length(intvec) != nrow(pre))
        stop("The number of intensity values does not match the dimension of the 'pre'.")
    actual.discrete <- max.col(actual, ties.method="first")
    pre.discrete <- max.col(pre, ties.method="first")

    #aclass <- unique(actual.discrete)
    #pclass <- unique(pre.discrete)
    #if (! (all(pclass %in% aclass)))
    #    stop("The range of the predicted classes is out of the range of the actual classes.")

    mse <- mean((pre - actual) ^ 2)
  
    pvolume <- unlist(lapply(1:ncol(actual), function(i) sum(pre.discrete==i)))
    avolume <- unlist(lapply(1:ncol(actual), function(i) sum(actual.discrete==i)))
    rseVolume <-  abs(pvolume - avolume) / avolume
                   
    misclass <- mean(actual.discrete != pre.discrete)

    DSM <- getDSM(actual.discrete, pre.discrete)

    conTable <- getConTable(actual.discrete, pre.discrete)

    if(! is.null(intvec)){
        if (!requireNamespace("lattice", quietly=TRUE))
            stop("The \"lattice\" package is required to plot the intensity densities")
        intensity <- rep(intvec, 2)
        class <- c(actual.discrete, pre.discrete)
        g <- rep(c("actual", "predicted"), each=length(intensity)/2)
        dp <- lattice::densityplot(~ intensity | factor(class), groups = g,
                    plot.points = FALSE, ref = TRUE,
                    auto.key = list(columns = 2),
                    layout=c(1,ncol(pre)))
        plot(dp)
    }

    list(mse=mse, misclass=misclass, rseVolume=rseVolume, DSM=DSM, conTable=conTable)
}

# measureMRI() compares a predicted fuzzy-membership matrix against a
# ground-truth one, returning several agreement measures.

actual <- matrix(c(1,0,0,
                    1,0,0,
                    0,1,0,
                    0,0,1), ncol=3, byrow=TRUE)

# a perfect prediction
perfect <- actual
res <- measureMRI(actual=actual, pre=perfect)
expect_equal(res$mse, 0)
expect_equal(res$misclass, 0)
expect_equal(res$rseVolume, c(0,0,0))
expect_equal(res$DSM, c(1,1,1))
expect_equal(res$conTable, diag(3))

# a prediction that is confident but wrong for one voxel (row 3: true
# class 2, predicted class 3)
pre <- matrix(c(1,0,0,
                 1,0,0,
                 0,0,1,
                 0,0,1), ncol=3, byrow=TRUE)
res2 <- measureMRI(actual=actual, pre=pre)
expect_equal(res2$misclass, 1/4)
expect_true(res2$mse > 0)
# class 2 (row 3 in 'actual') was never predicted, so its Dice similarity
# is 0
expect_equal(res2$DSM[2], 0)

# dimension mismatch is rejected
expect_error(measureMRI(actual=actual, pre=pre[1:3,]))

# intvec, when given, must be a plain vector matching the number of voxels
expect_error(measureMRI(intvec=matrix(1:8, 2), actual=actual, pre=pre))
expect_error(measureMRI(intvec=1:3, actual=actual, pre=pre))

# getConTable()/getDSM() are the (unexported) internal helpers behind
# measureMRI(), operating directly on discrete class labels
ct <- mritc:::getConTable(c(1,1,2,3), c(1,1,3,3))
expect_equal(dim(ct), c(3,3))
expect_equal(sum(ct), 3)  # one column per predicted class, each summing to 1
dsm <- mritc:::getDSM(c(1,1,2,3), c(1,1,3,3))
expect_equal(dsm[2], 0)   # class 2 never predicted
expect_equal(dsm[1], 1)   # class 1 perfectly predicted

expect_error(mritc:::getConTable(c(1,2), c(1,2,3)))
expect_error(mritc:::getConTable(c(1,2,3), c(1,2,4)))

#split one voxel into 8 subvoxels
#input is a mask
split18 <- function(mask){
  M <- array(rep(1,8), dim=rep(2,3))
  kronecker(mask, M)

}

#get the match up table of voxels and their subvoxels
#input
#  x: the mask
getPos18 <- function(mask){
  submask <- split18(mask==1)
  subpos <- which(submask==1)
  ind <- which(mask==1, arr.ind=T)
  d <- dim(mask)
  nx=d[1]; ny=d[2]; nz=d[3]
  v1 <- (ind[,3]-1)*8*nx*ny+(ind[,2]-1)*4*nx+(ind[,1]-1)*2+1
  v5 <- v1 + nx*ny*4
  pos <- cbind(v1, v1+1, v1+nx*2, v1+nx*2+1, v5, v5+1, v5+nx*2, v5+nx*2+1)
  matrix(match(as.vector(pos), subpos),  ncol=8)
}


makeMRIspatial <- function(mask, nnei, sub=FALSE, bias=FALSE){
    if(! nnei %in% c(6, 18, 26))
        stop("'nnei' has to be among (6, 18, 26)")
    if(nnei %in% c(4, 6)) nblocks <- 2
    else nblocks <- 8

    weights <- NULL
    weineighbors <- NULL

    if(sub==FALSE){
        neighbors <- getNeighborsMRI(mask, nnei)
        blocks <- getBlocksMRI(mask, nblocks)
        subvox=NULL
 
    }
    else{
        submask <- split18(mask)
        neighbors <- getNeighborsMRI(submask, nnei)
        blocks <- getBlocksMRI(submask, nblocks)
        subvox <- getPos18(mask)
    }
    
    if(bias==TRUE){
        weights <-getWeightsMRI(nnei=nnei, sigma=1)
        nvertex <- nrow(neighbors)
        nei <- (neighbors < nvertex+1)
        nei <- nei %*% weights
        weineighbors <-  rowSums(nei) 
    }
    
    result <- list(neighbors=neighbors, blocks=blocks, sub=sub, subvox=subvox,
                   weights=weights, weineighbors=weineighbors)
 
    result
}

#get possible configurations of a graph
getConfs <- function(n, nc){
    if(n < 2) stop("There are at least 2 vertices!")
    if(nc < 2) stop("There are at least two possible choices for each vertex!")

    subcc <- function(num){
        lapply(1:nc, function(x) list(c(num, x)))
    }

    cc <- function(n){
        if(n > 2){
            part <- matrix(unlist(cc(n-1)), nrow=n-1)
            lapply(1:ncol(part), function(x) subcc(part[,x]))
        }
        else{
            lapply(1:nc, function(x) subcc(x))
        }

    }

    matrix(unlist(cc(n)), nrow=n)
}
#get unique configurations 
getConfsUni <- function(nneigh, k){
    com <- getConfs(k, nneigh+1)
    com <- com - 1
    or <- order(apply(com, 2, function(x)
                      sum(sapply(1:k, function(i) x[i]*(nneigh+1)^(i-1)))))
    com[,or]
}

#compute the check table of probabilities for different neighbor configurations used in updating the indices based on a Potts model
getCheck <- function(nneigh, k, beta, spatialMat){
    if(k==3){
        if(nneigh==6) load(system.file("tables/tabn6k3.rda", package="mritc"))
        else
            if(nneigh==18) load(system.file("tables/tabn18k3.rda", package="mritc"))
            else
                if(nneigh==26) load(system.file("tables/tabn26k3.rda", package="mritc"))
                else com <- getConfsUni(nneigh, k)     
    }
    else{
        com <- getConfsUni(nneigh, k) 
    }
    exp(beta *   t(apply(com, 2, function(x) spatialMat%*%x)))
}

#check errors of input of tissue classi 
checkErrors <- function(prop=NULL, mu, sigma, err=NULL){
    if(is.null(prop)){
        k <- length(mu)
        if (k != length(sigma))
            stop("The dimensions of 'mu' and 'sigma' do NOT match.")
        if (!all(sigma > 0))
            stop("All 'sigma's have to be positive")
    }
    else{
        if (!(all(prop > 0) && all(prop < 1)))
            stop("'prop' has to be between 0 and 1")
        if (!isTRUE(all.equal(sum(prop), 1)))
            stop("Sum of 'prop' has to be 1")
        if (!is.null(mu) && !is.null(sigma)){
            k <- length(prop)
            if (!(k == length(mu) && k == length(sigma)))
                stop("The dimensions of 'prop', 'mu' and 'sigma' do NOT match.")
        }
    }
    if (!all(sigma > 0))
        stop("All 'sigma's have to be positive")
    if(! is.null(err))
        if (any(err < 0)) stop("All 'err's have to be positive.")

}

#Compute the density of all vertices.
getDen <- function(yunique, n.yunique, ymatch, mu, sigma){
    k <- length(mu)
    dy <- rep(yunique, each=k)
    dmu <- rep(mu, n.yunique)
    dsigma <- rep(sigma, n.yunique)
    den <-  dnorm(dy, mean = dmu, sd = dsigma)
    den <- matrix(den, ncol=k, byrow=T)
    den <- den[ymatch,]
    den
}

#compute the relative errors
relerr <- function(x, y){
    max(abs(x - y)) / (1 + max(abs(x), abs(y)))
}

#Initialize indices
initialIndices <- function(y, nvert, mu, sigma, k, sub, subvox=NULL, type="pure"){
    if(sub == FALSE){
        if(type == "pure")
            indices <- max.col(matrix(dnorm(rep(y, k), mean=rep(mu, each=nvert),
                                            sd=rep(sigma, each=nvert)), ncol=k))
        else{
            indices <- matrix(dnorm(rep(y, k-2),mean=rep(mu, each=nvert),
                          sd=rep(sigma, each=nvert)), ncol=k-2)
            indices <- round((indices[,1] + 3*indices[,2] + 5*indices[,3]) /rowSums(indices))
        }

    }
    else{
        nobs <- nvert / 8
        index <- max.col(matrix(dnorm(rep(y/8,k),
                                      mean=rep(mu, each=nobs),
                                      sd=rep(sigma, each=nobs)), ncol=k))
        indices <- rep(0, nvert)
        indices[subvox] <- rep(index, 8)
    }
    indices <- do.call(cbind, lapply(1:k, function(x) indices==x))
    rbind(indices, rep(0L,k))
 
}
 
#check whether to stop the interation or not and whether ouput the curren state
checkStopVerbose <- function(muold, mu, sigmaold, sigma, err, it, maxit,
                             verbose, propold=NULL, prop=NULL){
    flag <- 0
    remu <- relerr(muold, mu)
    resigma <- relerr(sigmaold, sigma)
    if(is.null(prop)){
        if (remu < err[1] && resigma < err[2] || it > maxit)
            flag <- 1
        if (verbose)
            cat(paste("Iteration ", it,
                      ": relative error of mu = ", signif(remu, 1),
                      "; sigma = ", signif(resigma, 1),
                      "\n", sep=""))
    }
    else{
        reprop <- relerr(propold, prop)
        if (remu < err[1] && resigma < err[2] && reprop < err[3] || it > maxit)
            flag <- 1
        if (verbose && it %% 10 == 0)
            cat(paste("Iteration ", it,
                      ": relative error of mu = ", signif(remu, 1),
                      "; sigma = ", signif(resigma, 1),
                      "; prop = ", signif(reprop, 1),
                      "\n", sep=""))
        
    }
    flag
}


    

# checkErrors() is an internal helper used throughout the package to
# validate mixture-model parameters (prop, mu, sigma) and convergence
# tolerances (err) before an algorithm starts iterating.

# valid input: no error
expect_silent(mritc:::checkErrors(prop=c(0.3,0.3,0.4), mu=c(1,2,3), sigma=c(1,1,1), err=1e-4))
expect_silent(mritc:::checkErrors(mu=c(1,2,3), sigma=c(1,1,1), err=1e-4))
expect_silent(mritc:::checkErrors(mu=c(1,2,3), sigma=c(1,1,1)))

# prop must lie strictly between 0 and 1
expect_error(mritc:::checkErrors(prop=c(0, 0.5, 0.5), mu=c(1,2,3), sigma=c(1,1,1)))
expect_error(mritc:::checkErrors(prop=c(1, 0, 0), mu=c(1,2,3), sigma=c(1,1,1)))

# prop must sum to 1
expect_error(mritc:::checkErrors(prop=c(0.2, 0.2, 0.2), mu=c(1,2,3), sigma=c(1,1,1)))

# prop, mu and sigma must all have matching lengths
expect_error(mritc:::checkErrors(prop=c(0.5,0.5), mu=c(1,2,3), sigma=c(1,1,1)))
expect_error(mritc:::checkErrors(prop=c(0.5,0.5), mu=c(1,2), sigma=c(1,1,1)))

# mu and sigma must have matching lengths when prop is not supplied
expect_error(mritc:::checkErrors(mu=c(1,2,3), sigma=c(1,1)))

# sigma must be strictly positive, with or without prop
expect_error(mritc:::checkErrors(prop=c(0.3,0.3,0.4), mu=c(1,2,3), sigma=c(1,-1,1)))
expect_error(mritc:::checkErrors(mu=c(1,2,3), sigma=c(1,0,1)))

# err (when supplied) must be non-negative
expect_error(mritc:::checkErrors(mu=c(1,2,3), sigma=c(1,1,1), err=c(1e-4,-1)))
expect_silent(mritc:::checkErrors(mu=c(1,2,3), sigma=c(1,1,1), err=c(1e-4,0)))

otsu <- function(y, m){
    yvals <- sort(unique(y))
    L <- length(yvals)
    per <- as.vector(table(y)) / length(y)
    
    P <- matrix(0, nrow=L, ncol=L)
    S <- matrix(0, nrow=L, ncol=L)
    H <- matrix(0, nrow=L, ncol=L)

    P[1,] <- cumsum(per)
    S[1,] <- cumsum(per * yvals[1:L])
    for(u in 2:L)
        for(v in u:L){
            P[u,v] <- P[1,v] - P[1,u-1]
            S[u,v] <- S[1,v] - S[1,u-1]
         }
    H <- S^2 / P
  
   
    x <- seq(L)
    n <- length(x)
    if (n -1 < m) 
        stop("The number of thresholds is larger than the unique values minus 1.")
    e <- 0
    h <- m
    a <- 1:m
    rule <- c(0, a, L)
    sigma2 <- sum(sapply(1:(m+1), function(i) H[rule[i]+1, rule[i+1]]))

    thresh <- yvals[a]
    nmmp1 <- n - m + 1
    mp1 <- m + 1
    while (a[1] != nmmp1) {
        if (e < n - h) {
            h <- 1
            e <- a[m]
            j <- 1
        }
        else {
            h <- h + 1
            e <- a[mp1 - h]
            j <- 1:h
        }
        a[m - h + j] <- e + j
        if(a[m] != L){
            rule <- c(0, a, L)
            new <- sum(sapply(1:(m+1), function(i) H[rule[i]+1, rule[i+1]]))
            if(new > sigma2){
                sigma2 <- new
                thresh <- yvals[a]
            }
        }
    }
    thresh
}

initThresh <- function(y, thresh) {
    z <- rowSums(outer(y, thresh, ">")) + 1
    classes <- sort(unique(z))
    list(prop = sapply(classes, function(i) mean(z == i)),
         mu = sapply(classes, function(i) mean(y[z == i])),
         sigma = sapply(classes, function(i) sd(y[z == i]))
         )
}

initOtsu <- function(y, m) {
    if (length(m) > 1) {
        warning(paste("Argument m has", length(m), "elements: only the first used"))
        m <- m[1]
    }
    if (length(y) - 1 < m)
        stop("The number of thresholds is too larger.")
    thresh <- otsu(y, m)
    initThresh(y, thresh)
}

initProp <- function(y, prop) {
    checkErrors(prop, mu=NULL, sigma=NULL, err=NULL)
    prop <- cumsum(prop[-length(prop)])
    thresh <- quantile(y, prop)
    initThresh(y, thresh)
}

# initOtsu() and initProp() provide initial (prop, mu, sigma) estimates for
# a normal mixture model, via Otsu multi-level thresholding or via
# user-specified proportions respectively.

set.seed(100)
y <- c(rnorm(200, 10, 1), rnorm(200, 20, 1), rnorm(200, 30, 1))

# initOtsu recovers three well-separated clusters
init <- initOtsu(y, 2)
expect_equal(length(init$prop), 3)
expect_equal(length(init$mu), 3)
expect_equal(length(init$sigma), 3)
expect_equal(sum(init$prop), 1)
expect_true(all(init$sigma > 0))
expect_equal(sort(init$mu), c(10, 20, 30), tolerance=1)

# initProp with balanced proportions on the same data recovers similar
# thresholds/means to initOtsu
initp <- initProp(y, c(1/3, 1/3, 1/3))
expect_equal(initp$mu, init$mu, tolerance=1e-6)
expect_equal(initp$prop, init$prop, tolerance=1e-6)

# initProp with unbalanced proportions still returns a valid partition
initp2 <- initProp(y, c(0.1, 0.8, 0.1))
expect_equal(sum(initp2$prop), 1)
expect_equal(length(initp2$mu), 3)

# a single, very skewed threshold in initProp is reflected in the recovered
# proportions
expect_equal(initp2$prop, c(0.1, 0.8, 0.1), tolerance=1e-6)

# m must leave at least one observation per implied class: with only one
# unique value in y, no threshold can be found
expect_error(initOtsu(rep(1, 10), 2))

# only the first element of 'm' is used, with a warning, if length(m) > 1
expect_warning(init3 <- initOtsu(y, c(2, 3)))
expect_equal(init3, initOtsu(y, 2))

# initProp delegates proportion validation to checkErrors()
expect_error(initProp(y, c(0.5, 0.5, 0.5)))
expect_error(initProp(y, c(0, 0.5, 0.5)))

# otsu(): internal multi-level threshold search. Two well-separated,
# equal-sized clusters should yield a single threshold that falls strictly
# between them, and classifies every point correctly.
yy <- c(rep(0, 50), rep(100, 50))
th <- mritc:::otsu(yy, 1)
expect_true(th >= 0 && th < 100)
expect_equal(sum(yy > th), 50)

# initThresh(): recovers per-class prop/mu/sigma from known threshold(s)
res <- mritc:::initThresh(yy, 50)
expect_equal(res$prop, c(0.5, 0.5))
expect_equal(res$mu, c(0, 100))
expect_equal(res$sigma, c(0, 0))

inte <- function(yunique, n.yunique, ymatch, mu, sigma, n){
    n.yunique <- length(yunique)
    alpha <- seq(0, 1, length=n+1)
    sigma2 <- sigma^2
    mu <- alpha * mu[1] + (1 - alpha) * mu[2]
    sigma <- sqrt(alpha^2 * sigma2[1] + (1 - alpha)^2 * sigma2[2])
    dn <- getDen(yunique, n.yunique, 1:n.yunique, mu, sigma)
    den <- apply(dn, 1, function(x) (sum(2 * x) - x[1] - x[n+1]) / (2 * n) )
    den <- den[ymatch]
    den

}

mritc.pvhmrfem <- function(y, neighbors, blocks,
                      spatialMat=matrix(c(2, 1, -1, -1, -1, 1, 2, 1, -1, -1,
                        -1, 1, 2, 1, -1, -1, -1, 1, 2, 1, -1, -1, -1, 1, 2),
                        ncol=5), beta=0.6,
                      mu, sigma, err=1e-4, maxit=20, verbose){

    checkErrors(mu=mu, sigma=sigma, err=err)

    if (length(err) < 2) err <- rep(err, length.out = 2)

    k <- length(mu) + 2
    nneigh <- ncol(neighbors)
    nblocks <- length(blocks)
    neighbors <- structure(as.integer(neighbors), dim = dim(neighbors))

    yunique <- sort(unique(y))
    n.yunique <- length(yunique)
    nvert <- length(y)
    ymatch <- match(y, yunique)


    muold <- rep(0,k-2)
    sigmaold <- rep(0,k-2)
    niter <- 10

    check <- getCheck(nneigh, k, beta, spatialMat)
    indices <- initialIndices(y, nvert, mu, sigma, k, sub=FALSE, type="partial")

    it <- 0
    repeat{
        it <- it + 1
        muold <- mu
        sigmaold <- sigma
        den.part <- getDen(yunique, n.yunique, ymatch, mu, sigma)
        den <- matrix(0, nrow=nvert, ncol=k)
        den[,1] <- den.part[,1]
        den[,3] <- den.part[,2]
        den[,5] <- den.part[,3]
        den[,2] <- inte(yunique, n.yunique, ymatch, mu[1:2], sigma[1:2], 100)
        den[,4] <- inte(yunique, n.yunique, ymatch, mu[2:3], sigma[2:3], 100)

        for(j in 1:niter){
            indices <- updateIndicesHMRFEM(neighbors, nneigh, blocks, nblocks, as.integer(k), indices, check, den)
        }
        prob  <- updateProbs(neighbors, nneigh, indices, den, k, check)
        mu <- updateMus(prob[,c(1,3,5)], y)
        sigma <- updateSds(prob[,c(1,3,5)], y, mu, nvert, k-2)
        flag <- checkStopVerbose(muold, mu, sigmaold, sigma, err, it, maxit, verbose)
        if(flag==1) break
    }

    list(prob=prob, mu=mu, sigma=sigma)
}

getBlocksMRI <- function(mask, nblocks){
    if(length(dim(mask)) != 3)
        stop("The 'mask' has to be of dimension 3.")
    if(! all(unique(as.vector(mask)) %in% 0:1))
        stop("The value of mask has to be either 1 or 0.")
    if(all(mask==0))
        stop("All voxels are outside the mask.")
    if(! nblocks %in% c(2,8))
        stop("The number of blocks has to be equal to either 2 or 8.")
    
    
    ind <- which(mask==1, arr.ind=T)
    x.mark <- ind[,1] %% 2
    y.mark <- ind[,2] %% 2
    z.mark <- ind[,3] %% 2
    
    if(nblocks==2){
        BW <- ifelse(rowSums(cbind(x.mark, y.mark, z.mark)) %% 2 == 1, 1, 0)
        lapply(1:2, function(x) which(BW == (x-1)))
    }
    
    else
      lapply(1:8, function(x) which((x.mark + y.mark*2 + z.mark*4) == (x-1)))
}

\name{summary.mritc}
\alias{summary.mritc}
\title{Summary Method for Class "mritc"}
\description{
  Summarize some information of an object of class "mritc". 
}
\usage{
   \method{summary}{mritc}(object, ...)
}
\arguments{
   \item{object}{an object of class "mritc".}
   \item{...}{any additional arguments.}
 }
\value{
  The function computes and returns some
  summary statistics of the object obtained from running the
  function \code{\link[mritc]{mritc}}.
}
\examples{
  T1 <- readMRI(system.file("extdata/t1.rawb.gz", package="mritc"),
                c(91,109,91), format="rawb.gz")
  mask <-readMRI(system.file("extdata/mask.rawb.gz", package="mritc"),
                 c(91,109,91), format="rawb.gz")
  tc.icm <- mritc(T1, mask, method="ICM")
  summary(tc.icm)
}
\keyword{methods}

\name{writeMRI}
\alias{writeMRI}
\title{Write an MR Image}
\description{Write an MR image into a file of different formats. 
}
\usage{
   writeMRI(data, file, header, format)
}
\arguments{
  \item{data}{MRI data in a three dimensional array or four
	dimensional array with the forth dimension equal to 1.
	It could be also an object of class "nifti" or "anlz" as
	defined in the package
	\code{oro.nifti}.}
   \item{file}{the name of the image file to be written out.}
   \item{header}{the header file. \code{header} is set as \code{NULL}
	 for backward compatibility.}
   \item{format}{the format of the image file. Right now only the
	 "Analyze", "NIfTI", and raw byte (unsigned with 1 byte per
	 element in the byte stream) gzip formats are supported.}
}
\details{
  Header file is not needed for the file of "Analyze" or "NIfTI"
  format anymore. 

  Files of "Analyze" format are written out through
  the function \code{\link[oro.nifti]{writeANALYZE}}.
  Files of "NIfTI" format are written out through
  the function \code{\link[oro.nifti]{writeNIfTI}}.
}
\seealso{
  \code{\link[oro.nifti]{writeANALYZE}},
  \code{\link[oro.nifti]{writeNIfTI}}
}
\value{
  Nothing is returned.
}
\examples{
  \dontrun{
  writeMRI(vol, file="vol.rawb.gz", header=NULL, format="rawb.gz")
  
  writeMRI(vol, file="vol", header=NULL, format="nifti")

  writeMRI(vol, file="vol", header=NULL, format="analyze")
  }
}
\keyword{utilities}

\name{readMRI}
\alias{readMRI}
\title{Read an MR Image into an Array}
\description{Read an MR image of different formats into an array. 
}
\usage{
   readMRI(file, dim, format)
}
\arguments{
   \item{file}{the name of the image file to be read in.}
   \item{dim}{the dimensions of the image. It is required for the
	 image of type \code{raw.gz}, in which \code{dim} is a vector
	 of length three specifying dimensions in x, y, and z directions.
	 The default is \code{NULL}.}
   \item{format}{the format of the image file. Right now only the
	 "Analyze", "NIfTI", and raw byte (unsigned with 1 byte per
	 element in the byte stream) gzip formats are supported.}
}
\value{
  An array with the appropriate dimensions containing the image
  volume. 
}
\details{
  The files of "Analyze" format are read in through the
  function \code{\link[oro.nifti]{readANALYZE}}.
  The files of "NIfTI" format are read in through
  the function\code{\link[oro.nifti]{readNIfTI}}. 
}
\seealso{
  \code{\link[oro.nifti]{readANALYZE}},
  \code{\link[oro.nifti]{readNIfTI}}
}
\examples{
  \dontrun{
  vol1 <- readMRI("t1.rawb.gz", c(91,109,91), "rawb.gz")
  
  vol2 <- readMRI("t1.nii.gz", format="nifti")

  vol3  <- readMRI("t1.nii", format="nifti")

  vol4 <- readMRI("t1-analyze.img", format="analyze")
  }
}
\keyword{utilities}

\name{print.mritc}
\alias{print.mritc}
\title{Print Method for Class "mritc"}
\description{
  Print out some information of an object of class "mritc". 
}
\usage{
   \method{print}{mritc}(x, ...)
}
\arguments{
   \item{x}{an object of class "mritc".}
   \item{...}{any additional arguments.}
 }
\value{
  The function computes and returns some summary statistics of the
  object obtained from running the function \code{\link[mritc]{mritc}}.
}
\examples{
  \donttest{
  T1 <- readMRI(system.file("extdata/t1.rawb.gz", package="mritc"),
                c(91,109,91), format="rawb.gz")
  mask <-readMRI(system.file("extdata/mask.rawb.gz", package="mritc"),
                 c(91,109,91), format="rawb.gz")
  tc.icm <- mritc(T1, mask, method="ICM")
  print(tc.icm)
  }
}
\keyword{methods}

\name{makeMRIspatial}
\alias{makeMRIspatial}
\title{Obtain Spatial Features of a Mask of an MR Image}
\description{Obtain various spatial features of an MR image, which are
  used in tissue classification.}
\usage{
   makeMRIspatial(mask, nnei, sub, bias)
}
\arguments{
   \item{mask}{three dimensional array. The voxels with value 1 are
     inside the mask; with value 0 are outside. We just focus on voxels
inside the mask.}
   \item{nnei}{the number of neighbors. Right now only 6, 18,
	 and 26 neighbors are supported. For a 3D image,
	 besides defining 6 neighbors in the x, y, and z directions,
	 one can add 12 diagonal neighbors in the x-y, x-z, and y-z
	 planes, and another 8 on the 3D diagonals. This leads to a
	 six neighbor structure, an eighteen neighbor structure, and
	 a twenty-six neighbor structure.}
   \item{sub}{logical; if \code{TRUE}, a new \code{mask}
	 which splits each voxel into
	 eight subvoxels is generated, and then obtain the neighbors and blocks
	 of subvoxels; otherwise obtain the neighbors and blocks at
	 the voxel level. The default if \code{FALSE}.}
   \item{bias}{logical; if \code{TRUE}, the spatial parameters for biased
	 field correction are calculated. The default if \code{FALSE}.}
}
\value{
  A list containing the following components:
  \item{neighbors}{a matrix, each row of which giving the neighbors of a
    voxel or subvoxel. The number of rows is equal to the number of
	(sub)voxels within the \code{mask} and the number of columns is the
	number of neighbors of each (sub)voxel. For the (sub)voxels on the
	boundaries, when one or more of their
	neighbors are missing, the missing are represented by the total
	number of (sub)voxels within the \code{mask} plus 1.}	
  \item{blocks}{the (sub)voxels within each block are mutually
	independent given the (sub)voxels in other blocks.}
  \item{sub}{logical; the same as the input \code{sub}.}
  \item{subvox}{if \code{sub} is \code{TRUE}, it is a matrix,
	with each row giving the eight subvoxels of a voxel;
	otherwise it is equal to \code{NULL}.}
  \item{weights}{if \code{bias} is \code{TRUE}, it is a vector
	of weights of neighbors of every voxel for bias field correction;
	otherwise it is equal to \code{NULL}. The default is \code{NULL}.}
  \item{weineighbors}{if \code{bias} is \code{TRUE}, it is a vector
	of sum of weights of neighbors for bias field correction, one
	element per voxel;
	otherwise it is equal to \code{NULL}. The default is \code{NULL}.}
}
\references{
  Dai Feng, Dong Liang, and Luke Tierney (2013)
  An unified Bayesian hierarchical model for MRI tissue classification
  \emph{Statistics in Medicine}

  Dai Feng (2008)
  Bayesian Hidden Markov Normal Mixture Models with Application to MRI
  Tissue Classification
  \emph{Ph. D. Dissertation, The University of Iowa} 
} 
\examples{
  mask <- array(1, dim=c(2,2,2))
  spa <- makeMRIspatial(mask, nnei=6, sub=FALSE)
  spa <- makeMRIspatial(mask, nnei=6, sub=TRUE)
  spa <- makeMRIspatial(mask, nnei=26, sub=TRUE, bias=TRUE)
}
\keyword{spatial}

\name{emnormmix}
\alias{emnormmix}
\title{Estimate the Parameters of a Normal Mixture Model Using the EM Algorithm }
\description{Use the EM Algorithm to estimate the parameters of a normal mixture model.}
\usage{
   emnormmix(y, prop, mu, sigma, err, maxit, verbose)
}
\arguments{
   \item{y}{vector of observations.}
   \item{prop}{vector of initial estimate of the proportions of
	 different components of a normal mixture model.
     }
	 \item{mu}{vector of initial estimate of the means of different
	   components of a normal mixture model.
     }
   \item{sigma}{vector of initial estimate of the standard deviations of
	 different components of a normal mixture model.}
   \item{err}{relative maximum error(s) used to decide when to stop the
     iteration. It could be a vector of length three corresponding to
     the relative maximum errors of the means, standard deviations, and
     proportions of all components of a normal mixture model. When it is
     a scalar, all have the same relative maximum error.}
   \item{maxit}{maximum number of iterations to perform.}
   \item{verbose}{logical. If \code{TRUE}, then indicate
	 the level of output as the algorithm runs after every 10 iterations.}
}
\details{It is tailor-made for the case when observations are from
  a finite set (MRI data for example). The table lookup method is used to speed up the
  computation.
}
\value{
  \item{prop}{a vector of estimated proportions of different components of a
  normal mixture model.}
 \item{mu}{a vector of estimated means of different components of a
  normal mixture model.}
  \item{sigma}{a vector of estimated standard deviations of different components of a
    normal mixture model.}
}
\examples{
  prop <- c(0.3, 0.3, 0.4)
  mu <- c(-10, 0, 10)
  sigma <- rep(1, 3)
  y<- floor(rnormmix(n=100000, prop, mu, sigma)[,1])
  initial <- initOtsu(y, 2)
  emnormmix(y=y, prop=initial$prop, mu=initial$mu, sigma=initial$sigma,
            err=1e-7, maxit=100, verbose=TRUE)
}
\keyword{utilities}

\name{initNormMix}
\title{Get the Initial Estimate of the Parameters of a Normal Mixture Model}
\alias{initOtsu}
\alias{initProp}
\description{Obtain initial estimation of proportions, means, and
  standard deviations of different components (tissue types for MRI)
  based on threshold values generated by Otsu's method implemented by
  a fast algorithm, or proportion of different components.}
\usage{
   initOtsu(y, m)
   initProp(y, prop)
}
\arguments{
  \item{y}{a vector of intensity values of an image.}
  \item{m}{number of classes (tissue types for MRI) minus 1.}
  \item{prop}{the initial estimate of proportion of different components.}
}
\details{The exhaustive search part of the function for Otsu's algorithm
  is adapted from \code{\link[utils]{combn}}.  For
  \code{\link[mritc]{initProp}}, the threshold values are
  quantiles based on the initial estimate of proportion of different
  components.
}
\value{
     \item{prop}{a vector of initial estimate of the proportions of
	   different components (tissue types for MRI).}
	 \item{mu}{a vector of initial estimate of the means of different
	   components (tissue types for MRI).}
	 \item{sigma}{a vector of initial estimates of the standard
	   deviations of different components (tissue types for MRI).}
}
\note{For \code{\link[mritc]{initOtsu}}, it supports any number of
  \code{m}. However, for MRI data, it can be slow if \code{m} is bigger
  than 3 even with the fast algorithm implemented, since
  the Otsu's algorithm uses an exhaustive search. But it should be
  fine with \code{m} equal to 2, which corresponds to the
  typical case in MRI classification with three major tissue types CSF, GM,
  and WM.}
\references{
  Nobuyuki Otsu (1979).
  A threshold selection method from gray-level histograms
  \emph{IEEE Transactions on Systems, Man and Cybernetics} \bold{vol. 9}, 62-66
  
  Ping-Sung Liao, Tse-Sheng Chen and Pau-Choo Chung (2001)  
  A Fast Algorithm for Multilevel Thresholding 
  \emph{Journal of Information Science and Engineering} \bold{vol. 17}, 713-727
} 
\examples{
  #Example 1
  prop <- c(.3, .4, .3)
  mu <- c(40,  90, 130)
  sigma <- c(4, 8, 4)
  y <- floor(rnormmix(n=100000, prop, mu, sigma)[,1])
  initOtsu(y, 2)
  initProp(y, prop)

  #Example 2
  T1 <- readMRI(system.file("extdata/t1.rawb.gz", package="mritc"),
                c(91,109,91), format="rawb.gz")
  mask <-readMRI(system.file("extdata/mask.rawb.gz", package="mritc"),
                 c(91,109,91), format="rawb.gz")
  initOtsu(T1[mask==1], 2)
  initProp(T1[mask==1], c(0.17, 0.48, 0.35))
}
\keyword{classif}

\name{mritc-package}
\alias{mritc-package}
\docType{package}
\title{MRI Tissue Classification Package}
\description{
  Use various methods to do MRI tissue classification.
}
\section{Introduction}{This package provides tools for MRI tissue
  classification using normal mixture models and hidden Markov normal
  mixture models (with the partial volume effect and intensity non-uniformity
  addressed) fitted by various methods.

  Magnetic resonance imaging (MRI) is used to
  identify the major tissues within a subject's brain. 
  Classification is usually based on a single image providing one
  measurement for each volume element, or voxel, in a discretization of
  the brain.  A simple model for MRI tissue classification views each voxel
  as homogeneous, belonging entirely to one of the three major tissue
  types (cerebrospinal fluid (CSF), gray matter (GM), and white matter
  (WM));  the intensity of voxels are thus normally
  distributed with means and variances depending on the tissue types
  of their voxels.
  The tissue types are not known and need to be identified from the
  image. The assumption that all tissue types are independent leads to
  a simple normal mixture model with parameters estimated by the EM
  algorithm and tissue types assigned using the Bayes classifier.
  
  Since nearby voxels tend to be of the same tissue type, a Markov
  random field model (a model from the Potts model family is used
  in this case) can be used to capture the spatial similarity of
  voxels by assigning homogeneity relationship among tissue types of
  neighboring voxels.
  Again, given the tissue types, the intensity of voxels are
  independently and normally distributed with means and
  variances depending on their tissue types.
  Furthermore, the Markov random field model defined on finite space
  is referred to as the hidden Markov model. Therefore the model
  combine the normal mixture part and the Potts model part is called
  the hidden Markov normal mixture model.
  This model can be fitted by the Iterated Conditional Mode algorithm,
  the Hidden Markov Random Field EM algorithm, or a Markov chain Monte
  Carlo approach. 

  A more realistic model than the one just described would take into
  account the fact that the volume elements are not homogeneous;
  while some may contain only one tissue type, others on the interface
  will contain two or possibly three different tissue types. This
  phenomenon is called the partial volume (PV) effect.
  One approach to address the PV effect is to introduce
  intermediate classes. Usually this is
  done by introducing two more classes: the combination of the CSF and
  the GM and the combination of the GM and the WM.
  Voxels containing WM and CSF are very rare and are ignored.
  This helps reduce confounding in estimation and a number of studies
  have used this approach.  Among these methods, the Gaussian partial volume
  hidden Markov random field models fitted by the modified EM algorithm
  appears to be more competitive in performance.
  A new approach to this problem is to construct a higher resolution
  image in which each voxel is divided into eight subvoxels.
  For each voxel
  the measured value is the sum of the unobserved measurements for the
  subvoxels. The subvoxels are in turn assumed to be homogeneous and
  follow the simpler model described above. 

  Intensity non-uniformity is an artifact that the signal intensity
  varies smoothly across an image. It is caused by combination
  and interaction of effects from the device, pulse sequence, and
  object. A commonly used approach to tackle it is to assume the 
  the measured signal is equal to true signal multiplied by bias field
  associated with the intensity non-uniformity plus some noise. 
  The bias field needs to be spatially smoothly varying and is modeled
  as either jointly normally distributed, or a linear combination of
  smooth spline or polynomial basis functions. Instead, we propose using
  a locally smoothed prior on the bias field.

  A Bayesian hierarchical model aiming at modeling the partial volume
  effect and intensity non-uniformity simultaneously was proposed.
  Instead of splitting the task into different steps, the framework
  harmoniously integrates several sub-models addressing different issues
  in the MRI classification, through specification of the likelihood function
  and prior distributions. This approach could provide more accurate
  tissue classification and also allow more effective estimation of the
  proportion of each voxel that belongs to each of the major tissue
  types.

  Besides brain image segmentation, the methods provided in this package can be used for 
  classification of other spatial data as well.
}
\section{Usage}{
  The function \code{\link[mritc]{readMRI}} and
  \code{\link[mritc]{writeMRI}} are I/O
  functions for MRI data. Right now, the "Analyze", "NIfTI", and raw
  byte (unsigned with 1 byte per element in the byte stream) gzip
  formats are supported.

  For each MR image, there has to be a corresponding array, \code{mask},
  with values 1 and 0. Voxels with value 1 are inside the brain and 0
  are outside.
  Tissue classification is conducted on voxels inside the brain.

  The functions \code{\link[mritc]{mritc.em}}, \code{\link[mritc]{mritc.icm}},
  \code{\link[mritc]{mritc.hmrfem}}, and
  \code{\link[mritc]{mritc.bayes}}
  are used
  to conduct the MRI tissue classification using the normal mixture
  model fitted by the EM algorithm, the hidden Markov normal
  mixture model at the voxel level fitted by the Iterated
  Conditional Mode algorithm, the Hidden Markov Random Field EM
  algorithm,
  or the Bayesian method (with or without the PV 
  or bias field correction).  The function
  \code{\link[mritc]{mritc.pvhmrfem}} is for classification using Gaussian
  partial volume hidden Markov random field models fitted by the
  modified EM algorithm.
  Different components of the normal mixture model correspond to
  different tissue types. The number of components is flexible,
  say using five components model to address the PV effect by
  \code{\link[mritc]{mritc.em}}, \code{\link[mritc]{mritc.icm}},
  \code{\link[mritc]{mritc.hmrfem}}, or \code{\link[mritc]{mritc.bayes}}.

  In order to use the previous functions, the parameters of the
  normal mixture model and the Potts model have to be specified.
  Some parameters can be obtained using the functions
  \code{\link[mritc]{initOtsu}} and \code{\link[mritc]{makeMRIspatial}}.
  There are default values for other parameters.

  The function \code{\link[mritc]{mritc}} integrates all methods together,
  provides a uniform platform with easier usage, and generates an object
  of class "mritc" , for which generic functions
  \code{\link[mritc]{print.mritc}}, \code{\link[mritc]{summary.mritc}},
  and \code{\link[mritc]{plot.mritc}} are provided. 
}
\section{Computation Issues}{
  To improve the speed, the table lookup method was used in various
  places; vectorized computation was used to take advantage of
  conditional independence. 
  Some computations are performed by \bold{C}
  code, and the \bold{OpenMP} is used to parallelize key loops in
  the \bold{C} code. Sparse matrix multiplication is adopted as well.
}
\keyword{package}

\name{plot.mritc}
\alias{plot.mritc}
\title{Plot Method for Class "mritc"}
\description{
  Visualize MRI tissue classification results. 
}
\usage{
   \method{plot}{mritc}(x, method=c("RNifti","misc3d"), ...)
}
\arguments{
   \item{x}{an object of class "mritc"}
   \item{method}{the visualisation backend to use. The default,
     \code{"RNifti"}, uses \code{\link[RNifti]{view}}; the alternative,
     \code{"misc3d"}, uses the three-plane Tk-based viewer
     \code{\link[misc3d]{slices3d}}, which requires the \pkg{tkrplot}
     \pkg{tcltk} packages to be installed and useable.}
   \item{...}{any additional arguments for the relevant viewer.}
}
\details{
  Allocate a voxel to the tissue type with the highest probability and
  then visualise the result.
}
\note{
  Prior to version 0.6.0, \code{"misc3d"} was the only viewer supported,
  but owing to its system requirements \code{"RNifti"} was added as a
  new default for better portability.
}
 \value{
   \code{NULL}, invisibly
}
\seealso{
  \code{\link[RNifti]{view}}, \code{\link[misc3d]{slices3d}}
}
\examples{
  T1 <- readMRI(system.file("extdata/t1.rawb.gz", package="mritc"),
                c(91,109,91), format="rawb.gz")
  mask <-readMRI(system.file("extdata/mask.rawb.gz", package="mritc"),
                 c(91,109,91), format="rawb.gz")
  tc.icm <- mritc(T1, mask, method="ICM")
  plot(tc.icm)
}
\keyword{methods}

\name{mritc}
\title{MRI Tissue Classification Using Various Methods}
\alias{mritc.em}
\alias{mritc.icm}
\alias{mritc.hmrfem}
\alias{mritc.pvhmrfem}
\alias{mritc.bayes}
\alias{mritc}
\description{
  Conduct the MRI tissue classification using different methods including:
  the normal mixture model (NMM) fitted by the Expectation-Maximization (EM)
  algorithm;
  the hidden Markov normal mixture model (HMNMM) fitted by the Iterated
  Conditional Mode (ICM) algorithm, the Hidden Markov
  Random Field EM (HMRFEM) algorithm, or the Bayesian Markov chain Monte
  Carlo method (MCMC); 
  the partial volume HMNMM fitted by the modified EM (PVHMRFEM)
  algorithm or the higher resolution HMNMM fitted by the MCMC method (MCMCsub);
  the HMNMM with both PV and intensity non-uniformity addressed (MCMCsubbias).
}
\usage{
   mritc.em(y, prop, mu, sigma, err, maxit, verbose)
   mritc.icm(y, neighbors, blocks, spatialMat, beta, mu, sigma,
             err, maxit, verbose)
   mritc.hmrfem(y, neighbors, blocks, spatialMat, beta, mu, sigma,
                err, maxit, verbose)
   mritc.pvhmrfem(y, neighbors, blocks, spatialMat, beta, mu, sigma,
              err, maxit, verbose)
   mritc.bayes(y, neighbors, blocks, sub, subvox,
               subbias, neighbors.bias, blocks.bias, weineighbors.bias, weights.bias,
               spatialMat, beta, mu, sigma, niter, verbose)
   mritc(intarr, mask, method, verbose = TRUE)
}
\arguments{
  \item{y}{a vector of intensity values of voxels.}
  \item{prop}{a vector of initial estimate of the proportions of
	different components of a normal mixture model. It can be obtained using
    the function \code{\link[mritc]{initOtsu}}.}
  \item{mu}{a vector of initial estimate of the means of different
	components of a normal mixture model. It can be obtained using
    the function \code{\link[mritc]{initOtsu}}.}
  \item{sigma}{a vector of initial estimates of the standard
	deviations of different components of a normal mixture model.
	It can be obtained using the function \code{\link[mritc]{initOtsu}}.}
  \item{err}{relative maximum error(s) used to decide when to stop the
    iteration. It could be a vector corresponding to
    the relative maximum errors of the means, standard deviations (for
	\code{\link[mritc]{mritc.em}}, \code{\link[mritc]{mritc.icm}},
	\code{\link[mritc]{mritc.hmrfem}},
	and \code{\link[mritc]{mritc.pvhmrfem}}), and
    proportions (for \code{\link[mritc]{mritc.em}})
	of all components of a normal mixture model. When it is
    a scalar, all have the same relative maximum error. The default
	value is 1e-4.
  }
  \item{maxit}{maximum number of iterations to perform. The default is
	200 for \code{\link[mritc]{mritc.em}}, 20 for
	\code{\link[mritc]{mritc.icm}}, \code{\link[mritc]{mritc.hmrfem}},
	and \code{\link[mritc]{mritc.pvhmrfem}}.} 
  \item{verbose}{logical. If \code{TRUE}, then indicate
	the level of output as the algorithm runs.}
  \item{neighbors}{a matrix of neighbors of voxels. One row per voxel.
	It can be obtained 
	using the function \code{\link[mritc]{makeMRIspatial}}.}
  \item{blocks}{split voxels into different blocks to use the
    checker-board idea. It can be obtained using the function
	\code{\link[mritc]{makeMRIspatial}}.}
  \item{spatialMat}{a matrix defining the spatial relationship
    in a Potts model. The default value is \code{diag(1,3)}
	for three components models for \code{\link[mritc]{mritc.icm}},
	\code{\link[mritc]{mritc.hmrfem}} and
	\code{\link[mritc]{mritc.bayes}} when \code{sub} is \code{FALSE}
	and \code{matrix(c(2,0,-1,0,2,0,-1,0,2), nrow=3)}
	when \code{sub} is \code{TRUE}.
	For \code{\link[mritc]{mritc.pvhmrfem}} the default is
	\code{matrix(c(2, 1, -1, -1, -1, 1, 2, 1, -1, -1, -1, 1,
	  2, 1, -1, -1, -1, 1, 2, 1, -1, -1, -1, 1, 2), ncol=5)}.}
  \item{beta}{the parameter 'inverse temperature' of the Potts
	model. The default value is 0.4 for \code{\link[mritc]{mritc.icm}},
	0.5 for \code{\link[mritc]{mritc.hmrfem}}, 0.6 for
	\code{\link[mritc]{mritc.pvhmrfem}}.
	For \code{\link[mritc]{mritc.bayes}}, the default is 0.7
	when \code{sub} is \code{FALSE} and 0.3 when \code{sub} is
	\code{TRUE}.}
  \item{sub}{logical; if \code{TRUE}, use the higher resolution model;
	otherwise, use the whole voxel method.}
  \item{subvox}{for \code{\link[mritc]{mritc.bayes}},
	the match up tabel of voxels and their corresponding
     subvoxels for the higher resolution model. It can be
	 obtained using the function \code{\link[mritc]{makeMRIspatial}}.
	 For the whole voxel method, \code{subvox}=\code{NULL}}
  \item{subbias}{logical; if \code{TRUE}, use the model that addresses
	both the PV and intensity non-uniformity. The default is \code{FALSE}.}
  \item{neighbors.bias}{a matrix of neighbors of bias field. One row per voxel.
	It can be obtained using the function
	\code{\link[mritc]{makeMRIspatial}}. The default is \code{NULL}.}
  \item{blocks.bias}{blocks for bias field. It can be obtained using the function
	\code{\link[mritc]{makeMRIspatial}}. The default is \code{NULL}.}
  \item{weineighbors.bias}{a vector of sum of weights of neighbors of bias
	field. One element per voxel. It can be obtained using the function
	\code{\link[mritc]{makeMRIspatial}}. The default is \code{NULL}.}
  \item{weights.bias}{a vector of weights of different neighbors of
	every voxel. It can be obtained using the function
	\code{\link[mritc]{makeMRIspatial}}. The default is \code{NULL}.}
  \item{niter}{the number of iterations for
	 \code{\link[mritc]{mritc.bayes}}. The default values are 1000 and
	 100 for with and without bias field correction, respectively.
	 The default values seem to be adequate in many cases.}
  \item{intarr}{a three dimensional array of an MR image.}
  \item{mask}{a mask of the MR image. Voxels with value 1 are
	inside the	brain and value 0 are outside. Focus on voxels
	within the brain.}
  \item{method}{a string giving the method for MRI tissue
	classification. It must be one of "EM", "ICM", "HMRFEM", 
	"MCMC", "PVHMRFEM", "MCMCsub", or "MCMCsubbias" corresponding to using the
	NMM fitted by the EM algorithm;
    the HMNMM fitted by the ICM algorithm, the HMRFEM algorithm,
	or the MCMC; the partial volume HMNMM fitted by the PVHMRFEM algorithm;
    the higher resolution HMNMM fitted by the MCMC; the HMNMN addressing
	both the PV and intensity non-uniformity.
    It can be abbreviated. The default is "EM".
  }
}
\details{
  The function \code{\link[mritc]{mritc}} integrates functions
  \code{\link[mritc]{mritc.em}},
  \code{\link[mritc]{mritc.icm}}, \code{\link[mritc]{mritc.hmrfem}}, 
  \code{\link[mritc]{mritc.pvhmrfem}}, and \code{\link[mritc]{mritc.bayes}}.
  It provides a uniform platform with easier
  usage. The user just need to specify the input MR image, the
  \code{mask} of the image, and the \code{method} used.
  The other parameters are specified automatically as follows.
  The parameters for the initial estimates of the proportions,
  means, and standard deviations of the normal mixture model are
  obtained using the function \code{\link[mritc]{initOtsu}}.
  As to the parameters related to the Potts model,
  the six neighbor structure is used and then the \code{neighbors},
  \code{blocks}, and \code{subvox} are
  obtained using the function \code{\link[mritc]{makeMRIspatial}}.
  For the bias field correction, the twenty-six neighbor structure is
  used and then the \code{neighbors.bias}, \code{blocks.bias},
  \code{weineighbors.bias} and \code{weights.bias} are
  obtained using the function \code{\link[mritc]{makeMRIspatial}}.
  The other parameters are taken as the default values
  for each method. The process is reported during iterations.
  
}
\note{
  The functions support the various normal mixture models with
  at most eight components.
 }
\value{
  For \code{\link[mritc]{mritc}}, it generates
  an object of class "mritc" which is a list containing the
  following components:
   \item{prob}{a matrix, one row per voxel and each column
	corresponding to the probabilities of being allocated to each
	component of a normal mixture model.}
  \item{mu}{a vector of estimated means of the normal mixture model.}
  \item{sigma}{a vector of estimated standard deviations of the
	 normal mixture model.}
   \item{method}{the method used for computation.}
   \item{mask}{mask of an brain. Voxels inside it are classified.}
   
   Generic functions \code{\link[mritc]{print.mritc}}, 
   \code{\link[mritc]{summary.mritc}}, and
   \code{\link[mritc]{plot.mritc}} are provided.

   For others, only \code{prob}, \code{mu}, and \code{sigma} are
   generated.
 }
\references{
  Julian Besag (1986)
  On the statistical analysis of dirty pictures (with discussion)
  \emph{Journal of the Royal Statistical Society. Series B
	(Methodological)}
  \bold{vol. 48} 259-302
  
  Meritxell Bach Cuadra, Leila Cammoun, Torsten Butz, Olivier Cuisenaire,
  and Jean-Philippe Thiran (2005)
  Comparison and validation of tissue modelization and statistical
  classification methods in T1-weighted {MR} brain images
  \emph{IEEE Transactions on Medical Imaging}, \bold{vol.24} 1548-1565

  Dai Feng, Dong Liang, and Luke Tierney (2014)
  An unified Bayesian hierarchical model for MRI tissue classification
  \emph{Statistics in Medicine}
  \bold{vol.33, issue 8} 1349-1368
  
  Dai Feng, Luke Tierney, and Vincent Magnotta (2012)
  MRI tissue classification using high resolution
  Bayesian hidden Markov normal mixture models
  \emph{Journal of the American Statistical Association},
  \bold{vol.107, no.497} 102-119

  Dai Feng and Luke Tierney (2011)
  mritc: A package for MRI tissue classification
  \emph{Journal of Statistical Software}, \bold{vol.44, no.7} 1-20
  \url{https://www.jstatsoft.org/v44/i07/}

  Dai Feng (2008)
  Bayesian hidden Markov normal mixture models with application to MRI
  tissue classification
  \emph{Ph. D. Dissertation, The University of Iowa}

  Yongyue Zhang, Michael Brady, and Stephen Smith (2001)
  Segmentation of brain MR images through a hidden Markov random field
  model and the expectation-maximization algorithm
  \emph{IEEE Transactions on Medical Imaging} \bold{vol. 20} 45-57
 
} 
\examples{
  #Example 1
  T1 <- readMRI(system.file("extdata/t1.rawb.gz", package="mritc"),
                c(91,109,91), format="rawb.gz")
  mask <- readMRI(system.file("extdata/mask.rawb.gz", package="mritc"),
                  c(91,109,91), format="rawb.gz")
  y <- T1[mask==1]
  initial <- initOtsu(y, 2)
  prop <- initial$prop
  mu <- initial$mu
  sigma <- initial$sigma
  tc.em <- mritc.em(y, prop, mu, sigma, verbose=TRUE)
 
  \dontrun{
  mrispatial <- makeMRIspatial(mask, nnei=6, sub=FALSE)
  tc.icm <- mritc.icm(y, mrispatial$neighbors, mrispatial$blocks,
                      mu=mu, sigma=sigma, verbose=TRUE)
  tc.hmrfem <- mritc.hmrfem(y, mrispatial$neighbors, mrispatial$blocks,
                            mu=mu, sigma=sigma, verbose=TRUE)
  tc.pvhmrfem <- mritc.pvhmrfem(y, mrispatial$neighbors, mrispatial$blocks,
                                mu=mu, sigma=sigma, verbose=TRUE)
  tc.mcmc <- mritc.bayes(y, mrispatial$neighbors, mrispatial$blocks,
                         mrispatial$sub, mrispatial$subvox,
                         mu=mu, sigma=sigma, verbose=TRUE)

  mrispatial <- makeMRIspatial(mask, nnei=6, sub=TRUE)
  tc.mcmcsub <- mritc.bayes(y, mrispatial$neighbors, mrispatial$blocks,
                         mrispatial$sub, mrispatial$subvox,
                         mu=mu, sigma=sigma, verbose=TRUE)

  mrispatial26 <- makeMRIspatial(mask, nnei=26, sub=TRUE, bias=TRUE)
  tc.mcmcsubbias <- mritc.bayes(y, mrispatial$neighbors, mrispatial$blocks,
                                mrispatial$sub, mrispatial$subvox,
                                subbias=TRUE, mrispatial26$neighbors,
                                mrispatial26$blocks,mrispatial26$weineighbors,
                                mrispatial26$weights, mu=mu, sigma=sigma, verbose=TRUE)

  }
  #Example 2
  T1 <- readMRI(system.file("extdata/t1.rawb.gz", package="mritc"),
                c(91,109,91), format="rawb.gz")
  mask <-readMRI(system.file("extdata/mask.rawb.gz", package="mritc"),
                 c(91,109,91), format="rawb.gz")
  tc.icm <- mritc(T1, mask, method="ICM")
}
\keyword{classif}

\name{measureMRI}
\alias{measureMRI}
\title{Compare the Predicted Classsification Results with the Truth}
\description{Calculate and demonstrate different measures for
  classification results based on the truth.}
\usage{
   measureMRI(intvec, actual, pre)
}
\arguments{
  \item{intvec}{a vector of intensity values. If it is not \code{NULL},
	the density plots of each component corresponding to the actual and
	predicted classification results are shown. The default is
	\code{NULL}.}
   \item{actual}{matrix of the true classification result. Each row
	 corresponds to one voxel. Column \eqn{i} represents the
	 probabilities that all voxels are allocated to tissue type \eqn{i}.}
   \item{pre}{matrix of the predicted classification result. Each row
	 corresponds to one voxel. Column \eqn{i} represents the
	 probabilities that all voxels are allocated to tissue type \eqn{i}.
   }
}
\value{
  \item{mse}{mean square error.}
  \item{misclass}{mis-classification rate.}
  \item{rseVolume}{root square error of volume
	with respect to reference tissue volume.}
  \item{DSM}{Dice Similary Measure of each tissue type.
	\deqn{
  DSM_{a,b}^{t}=\frac{2 \times N_{a \cap b}^t}{N_a^t+N_b^t}  
    }
where \eqn{N_a^t} and \eqn{N_b^t} are the number of voxels classified as
tissue type \eqn{t} by
method \eqn{a} and \eqn{b} respectively, and \eqn{N_{a \cap b}^t} is the number of voxels
classified as tissue type \eqn{t} by both methods \eqn{a} and \eqn{b}.  The larger the
DSM, the more
similar the results from the two methods.
}
  \item{conTable}{confusion table. Each column of the table represents
	the instances in an actual class, while each row represents the
	instances in a predicted class.}
}
\examples{
  #Example 1
  prop <- c(.3, .4, .3)
  mu <- c(-4, 0, 4)
  sigma <- rep(1, 3)
  y <- rnormmix(n=1e4, prop, mu, sigma)
  intvec <- y[,1]
  actual <- y[,2]
  pre <- actual
  pre[sample(1:1e4, 100, replace=FALSE)]  <- sample(1:3, 100, replace=TRUE)
  actual <- do.call(cbind, lapply(1:3, function(i) ifelse(actual==i, 1, 0)))
  pre <- do.call(cbind, lapply(1:3, function(i) ifelse(pre==i, 1, 0)))
  measureMRI(intvec, actual, pre)

   \donttest{
  #Example 2
  T1 <- readMRI(system.file("extdata/t1.rawb.gz", package="mritc"),
                c(91,109,91), format="rawb.gz")
  mask <-readMRI(system.file("extdata/mask.rawb.gz", package="mritc"),
                 c(91,109,91), format="rawb.gz")
  tc.icm <- mritc(T1, mask, method="ICM")

  csf <- readMRI(system.file("extdata/csf.rawb.gz", package="mritc"),
                c(91,109,91), format="rawb.gz")
  gm <- readMRI(system.file("extdata/gm.rawb.gz", package="mritc"),
                c(91,109,91), format="rawb.gz")
  wm <- readMRI(system.file("extdata/wm.rawb.gz", package="mritc"),
                c(91,109,91), format="rawb.gz")
  truth <- cbind(csf[mask==1], gm[mask==1], wm[mask==1])
  truth <- truth/255
  measureMRI(T1[mask==1], truth, tc.icm$prob)
  }
}
\keyword{utilities}

\name{rnormmix}
\alias{rnormmix}
\title{Generate Random Samples from a Normal Mixture Model}
\description{Generate random samples from a normal mixture model.}
\usage{
   rnormmix(n, prop, mu, sigma)
}
\arguments{
   \item{n}{number of observations.}
   \item{prop}{a vector of proportions of different components.}
   \item{mu}{a vector of means of different components.}
   \item{sigma}{a vector of standard deviations of different components}
}
\value{
  A matrix with each row corresponding to one sample. The first column are
  sample values from a normal mixture model; the second column are the
  components from which observations come.
}
\examples{
  prop <- c(.17, .48, .35)
  mu <- c(-4, 0, 4)
  sigma <- rep(1, 3)
  y <- rnormmix(n=10000, prop, mu, sigma)
  hist(y[,1], breaks=50)
}
\keyword{distribution}
